*! version 1.2 7August 15, 2007 @ 14:07:36

program define cleanbank

   version 7.0

   syntax varlist(string) [if] [in], [FULLnames     /* unabbreviate all descriptor words
   */                                 REMove        /* remove all descriptor words
   */                                 WORDS(string) /* user-specified words to remove from varlist
   */                                 PAREN         /* remove parenthetical expressions
   */                                 states        /* follow `fullnames' behavior for statenames
                                                        (default is do not alter state names)
   */                                 noSPACES]     /* remove all spaces from varlist */



   /*todo: add state abbreviations
           behavior to turn-on by option states, then follows fullnames behavior for
           abbreviate or expand.
     todo: add international bank (and company) abbreviations (N.V., SpA, PLC)
     todo: replace "&" with "AND"
     todo: improve symbol removal and accent replacement using ascii keymap. */
   

   if `"`words'"' != "" & "`remove'" != "remove" {
      display as error "option words() not allowed unless option remove specified"
      exit 198
   }

   if `"`fullnames'"' == "fullnames" & "`remove'" == "remove" {
      display as error "option fullnames not allowed with option remove"
      exit 198
   }

   if "`paren'"=="paren" {
      if "$S_StataSE"=="SE" {
         local strlen = 244
      }
      else {
         local strlen = 80
      }
      tempvar i1 i2
   }


   tempvar touse
   
   local symbols `"~ ! @ # $ % ^ & * ( ) - _ = + [ { ] } \ | ; : , < . > / ?"'
   local abbwords `"INC CORP CO FSB NA NB GROUP GRP HOLDINGS HLDGS LTD LLC ADR"'  /*abbreviations*/
   local arwords `"AND THE OF"' /*always remove words*/
   local srwords `"AMERN BK NY PPTYS PPTY GP RLTY CDN COML PRODS INTL CMNTY CTZNS COMMS ENGY PRODS TR INDS MLS ELEC ENTMT MGMT MTN MACHS WKS NATL FINL AIRLS SVCS NTHN STHN CTR STD UNVL MATL FIRST UTD STS FED BANCORP CORP CO INC"' /*somteimes remove words - not currently used*/
   local words = upper(`"`words'"') /*user-specified remove words*/


   preserve

   quietly {

      generate `touse' = 0
      replace `touse' = 1 `in' `if'

      foreach var of local varlist {
      
         /*Optional: Remove parenthetical expressions*/
         if "`paren'"=="paren" {

            generate `i1' = index(`var', "(")
            generate `i2' = index(`var', ")")

            replace `var' = substr(`var', 1, `i1'-1) + " " + /*
               */ substr(`var', `i2'+1, .) /*
               */ if `i1'!=0 & `i2'!=0 & `touse'
            drop `i1' `i2'
         }


         /* Always perform these actions */

            /*remove european characters*/
         forvalues i = 192/255 {
            if `i' >=224 & `i'<=229 {
               replace `var' = subinstr(`var', char(`i'), "a", .) if `touse'
            }
            if `i' ==230 {
               replace `var' = subinstr(`var', char(`i'), "ae", .) if `touse'
            }
            if `i' ==231 {
               replace `var' = subinstr(`var', char(`i'), "c", .) if `touse'
            }
            if `i' >=232 & `i'<=235 {
               replace `var' = subinstr(`var', char(`i'), "e", .) if `touse'
            }
            if `i' >=236 & `i'<=239 {
               replace `var' = subinstr(`var', char(`i'), "i", .) if `touse'
            }
            if `i' ==240 | (`i'>=242 & `i'<=246) {
               replace `var' = subinstr(`var', char(`i'), "o", .) if `touse'
            }
            if `i' == 241 {
               replace `var' = subinstr(`var', char(241), "n", .) if `touse'
            }
            if `i' >=249 & `i'<=252 {
               replace `var' = subinstr(`var', char(`i'), "u", .) if `touse'
            }
         }

           /*uppercase the name*/
         replace `var' = upper(`var') if `touse'

           /*prefer "first"*/
         replace `var' = subinword(`var', "1ST", "FIRST", .) if `touse'

           /*remove symbols*/
         foreach symbol of local symbols {
            replace `var' = subinstr(`var',`"`symbol'"'," ",.) if `touse'
         }
           /*no-space for right apostrophe*/
         replace `var' = subinstr(`var', `"'"', "", .) if `touse'

           /* replace "National Association" with "NA" */
         replace `var' = subinstr(`var', "NATIONAL ASSOCIATION", "NA",.) if `touse'
         replace `var' = subinstr(`var', "NATL ASSOCIATION", "NA",.) if `touse'

           /*todo: we would like to remove left apostrophe and double-quote here */

           /*always-remove words*/
         foreach arword of local arwords {
            replace `var' = subinword(`var', "`arword'", "", .)
         }


         /* Optional behavior: unabbreviate all names */
         if "`fullnames'"=="fullnames" {
            replace `var' = subinword(`var', "AMERN", "AMERICAN", .)
            replace `var' = subinword(`var', "BK", "BANK", .)
            replace `var' = subinword(`var', "NY", "NEW YORK", .)
            replace `var' = subinword(`var', "PPTYS", "PROPERTIES", .)
            replace `var' = subinword(`var', "PPTY", "PROPERTY", .)
            replace `var' = subinword(`var', "GP", "GROUP", .)
            replace `var' = subinword(`var', "RLTY", "REALTY", .)
            replace `var' = subinword(`var', "CDN", "CANADIAN", .)
            replace `var' = subinword(`var', "COML", "COMMERCIAL", .)
            replace `var' = subinword(`var', "PRODS", "PRODUCTS", .)
            replace `var' = subinword(`var', "INTL", "INTERNATIONAL", .)
            replace `var' = subinword(`var', "CMNTY", "COMMUNITY", .)
            replace `var' = subinword(`var', "CTZNS", "CITIZENS", .)
            replace `var' = subinword(`var', "COMMS", "COMMUNICATIONS", .)
            replace `var' = subinword(`var', "ENGY", "ENERGY", .)
            replace `var' = subinword(`var', "PRODS", "PRODUCTS", .)
            replace `var' = subinword(`var', "TR", "TRUST", .)
            replace `var' = subinword(`var', "INDS", "INDUSTRIES", .)
            replace `var' = subinword(`var', "MLS", "MILLS", .)
            replace `var' = subinword(`var', "ELEC", "ELECTRIC", .)
            replace `var' = subinword(`var', "ENTMT", "ENTERTAINMENT", .)
            replace `var' = subinword(`var', "MGMT", "MANAGEMENT", .)
            replace `var' = subinword(`var', "MTN", "MOUNTAIN", .)
            replace `var' = subinword(`var', "MACHS", "MACHINES", .)
            replace `var' = subinword(`var', "WKS", "WORKS", .)
            replace `var' = subinword(`var', "NATL", "NATIONAL", .)
            replace `var' = subinword(`var', "FINL", "FINANCIAL", .)
            replace `var' = subinword(`var', "AIRLS", "AIRLINES", .)
            replace `var' = subinword(`var', "SVCS", "SERVICES", .)
            replace `var' = subinword(`var', "NTHN", "NORTHERN", .)
            replace `var' = subinword(`var', "STHN", "SOUTHERN", .)
            replace `var' = subinword(`var', "CTR", "CENTER", .)
            replace `var' = subinword(`var', "STD", "STANDARD", .)
            replace `var' = subinword(`var', "UNVL", "UNIVERSAL", .)
            replace `var' = subinword(`var', "MATL", "MATERIAL", .)
            replace `var' = subinword(`var', "UTD", "UNITED", .)
            replace `var' = subinword(`var', "STS", "STATES", .)
            replace `var' = subinword(`var', "THE", "", .)
            replace `var' = subinword(`var', "OF", "", .)
            replace `var' = subinword(`var', "FED", "FEDERAL", .)
            replace `var' = subinword(`var', "BANCORP", "BANCORPORATION", .)
            replace `var' = subinword(`var', "CORP", "CORPORATION", .)
            replace `var' = subinword(`var', "CO", "COMPANY", .)
            replace `var' = subinword(`var', "INC", "INCORPORATED", .)
            replace `var' = subinword(`var', "HLDGS", "HOLDINGS", . )
            replace `var' = subinword(`var', "LTD", "LIMITED", . )            
         }
         else {
            /* Default behavior: abberviate all names */
            replace `var' = subinword(`var', "AMERICAN", "AMERN", .)
            replace `var' = subinword(`var', "BANK", "BK", .)
            replace `var' = subinword(`var', "NEW YORK", "NY", .)
            replace `var' = subinword(`var', "PROPERTIES", "PPTYS", .)
            replace `var' = subinword(`var', "PROPERTY", "PPTY", .)
            replace `var' = subinword(`var', "GROUP", "GP", .)
            replace `var' = subinword(`var', "REALTY", "RLTY", .)
            replace `var' = subinword(`var', "CANADIAN", "CDN", .)
            replace `var' = subinword(`var', "COMMERCIAL", "COML", .)
            replace `var' = subinword(`var', "PRODUCTS", "PRODS", .)
            replace `var' = subinword(`var', "INTERNATIONAL", "INTL", .)
            replace `var' = subinword(`var', "COMMUNITY", "CMNTY", .)
            replace `var' = subinword(`var', "CITIZENS", "CTZNS", .)
            replace `var' = subinword(`var', "COMMUNICATIONS", "COMMS", .)
            replace `var' = subinword(`var', "ENERGY", "ENGY", .)
            replace `var' = subinword(`var', "PRODUCTS", "PRODS", .)
            replace `var' = subinword(`var', "TRUST", "TR", .)
            replace `var' = subinword(`var', "INDUSTRIES", "INDS", .)
            replace `var' = subinword(`var', "MILLS", "MLS", .)
            replace `var' = subinword(`var', "ELECTRIC", "ELEC", .)
            replace `var' = subinword(`var', "ENTERTAINMENT", "ENTMT", .)
            replace `var' = subinword(`var', "MANAGEMENT", "MGMT", .)
            replace `var' = subinword(`var', "MOUNTAIN", "MTN", .)
            replace `var' = subinword(`var', "MACHINES", "MACHS", .)
            replace `var' = subinword(`var', "WORKS", "WKS", .)
            replace `var' = subinword(`var', "NATIONAL", "NATL", .)
            replace `var' = subinword(`var', "FINANCIAL", "FINL", .)
            replace `var' = subinword(`var', "AIRLINES", "AIRLS", .)
            replace `var' = subinword(`var', "SERVICES", "SVCS", .)
            replace `var' = subinword(`var', "NORTHERN", "NTHN", .)
            replace `var' = subinword(`var', "SOUTHERN", "STHN", .)
            replace `var' = subinword(`var', "CENTER", "CTR", .)
            replace `var' = subinword(`var', "STANDARD", "STD", .)
            replace `var' = subinword(`var', "UNIVERSAL", "UNVL", .)
            replace `var' = subinword(`var', "MATERIAL", "MATL", .)
            replace `var' = subinword(`var', "UNITED", "UTD", .)
            replace `var' = subinword(`var', "STATES", "STS", .)
            replace `var' = subinword(`var', "FEDERAL", "FED", .)
            replace `var' = subinword(`var', "BANCORPORATION", "BANCORP", .)
            replace `var' = subinword(`var', "CORPORATION", "CORP", .)
            replace `var' = subinword(`var', "COMPANY", "CO", .)
            replace `var' = subinword(`var', "INCORPORATED", "INC", .)
            replace `var' = subinword(`var', "HOLDINGS", "HLDGS", . )
            replace `var' = subinword(`var', "LIMITED", "LTD", . )
         }
         
         if "`remove'"=="remove" {
            /* Optional behavior: remove all abbreviations and user-specified words */
            foreach abbword of local abbwords {
               replace `var' = subinword(`var',`"`abbword'"'," ",.) if `touse'
            }
            foreach word of local words {
               replace `var' = subinword(`var',`"`word'"'," ",.) if `touse'
            }
         }
         


         

         * either cut out all spaces, or just the extra ones between words (default)
         if "`spaces'"=="nospaces" {
            replace `var' = subinstr(`var'," ","",.) if `touse'
         }
         else {
            tempvar length
            generate `length' = length(`var')
            summarize `length', meanonly
            local l2 = r(max)/2
            forvalues n = 1/`l2' {
               replace `var' = subinstr(`var',"  "," ",.)
            }
            replace `var' = trim(`var')
         }
         
      } * ends variable loop

   } * ends quietly block
   
   restore, not

end
