*! version 2.19, Ben Jann, 20dec2005

program define estout, rclass
	version 8.2
	syntax [anything] [using] [ , ///
	 Cells(string asis) ///
	 Drop(passthru)  ///
	 Keep(passthru) ///
	 EQuations(passthru) ///
	 NOEFORM eform EFORM2(string) ///
	 NOMargin Margin Margin2(string) ///
	 NODIscrete DIscrete(string asis) ///
	 MEQs(string) ///
	 level(numlist max=1 int >=10 <=99) ///
	 Stats(string asis) ///
	 STARLevels(string asis) ///
	 NOSTARDetach STARDetach ///
	 VARwidth(numlist max=1 int >=0) ///
	 MODELwidth(numlist max=1 int >=0) ///
	 NOABbrev ABbrev ///
	 NOUNStack UNStack ///
	 BEGin(string asis) ///
	 DELimiter(string asis) ///
	 end(string asis) ///
	 DMarker(string) ///
	 MSign(string) ///
	 NOLZ lz ///
	 SUBstitute(string asis) ///
	 TItle(string) ///
	 NOLEgend LEgend ///
	 PREHead(string asis) ///
	 POSTHead(string asis) ///
	 PREFoot(string asis) ///
	 POSTFoot(string asis) ///
	 NOLabel Label ///
	 VARLabels(string asis) ///
	 MLabels(string asis) ///
	 COLLabels(string asis) ///
	 EQlabels(string asis) ///
	 MGRoups(string asis) ///
	 NOReplace Replace ///
	 NOAppend Append ///
	 NOTYpe TYpe ///
	 NOSHOWTABS showtabs ///
	 STYle(string) ///
	 DEFaults(string) ///
	 ]
	if "`style'"!="" local defaults "`style'"

*Parse suboptions
	if `"`cells'"'!="none" {
		gettoken row rest: cells, bind qed(qed)
		local cells
		while `"`row'"'!="" {
			local newrow
			gettoken opt row: row, parse(" (")
			gettoken trash row0: row, match(par)
			gettoken opt2: row, match(par)
			while "`opt'"!="" {
				local newrow: list newrow | opt
				if `"`par'"'!="(" {
					gettoken opt row: row, parse(" (")
				}
				else {
					ParseValueSubopts `opt', `macval(opt2)'
					gettoken opt row: row0, parse(" (")
				}
				gettoken trash row0: row, match(par)
				gettoken opt2: row, match(par)
			}
			if `qed' local cells `"`cells'"`newrow'" "'
			else local cells `"`cells'`newrow' "'
			gettoken row rest: rest, bind qed(qed)
		}
		local cells: list retok cells
	}
	if "`eform2'"!="" {
		foreach p of local eform2 {
			if !( "`p'"=="1" | "`p'"=="0" ) {
				di as error `""`eform2'" invalid in eform()"'
				exit 198
			}
		}
		local eform "`eform2'"
		local eform2
	}
	if "`margin2'"!="" {
		local margin "`margin2'"
		local margin2
	}
	if `"`macval(stats)'"'!="" {
		ParseStatsSubopts `macval(stats)'
		if `"`macval(statslabels)'"'!="" {
			ParseLabelsSubopts statslabels `macval(statslabels)'
		}
	}
	foreach opt in mgroups mlabels eqlabels collabels varlabels {
		if `"`macval(`opt')'"'!="" {
			ParseLabelsSubopts `opt' `macval(`opt')'
		}
	}

*Process No-Options
	foreach opt in unstack eform margin discrete stardetach ///
	 legend label lz abbrev replace append type showtabs {
		if "`no`opt''"!="" local `opt'
	}

*Read defaults
	if "`defaults'"=="" {
		if `"`macval(delimiter)'"'=="" local delimiter _tab
		if "`notype'"==""              local type type
		if "`nolz'"==""                local lz lz
		if "`nostatslabelslast'"==""   local statslabelslast last
		if "`novarlabelslast'"==""     local varlabelslast last
		if `"`macval(discrete)'"'=="" & "`nodiscrete'"=="" {
			local discrete `"" (d)" marginals for discrete change of dummy variable from 0 to 1"'
		}
	}
	else {
		capture findfile estout_`defaults'.def
		if _rc {
			di as error `"`defaults' defaults not available "' ///
			 `"(file estout_`defaults'.def not found)"'
			exit 601
		}
		else {
			tempname file
			file open `file' using `"`r(fn)'"', read text
			if c(SE) local max 244
			else local max 80
			while 1 {
				ReadLine `max' `file'
				if `"`line'"'=="" continue, break
				gettoken opt line: line
				if `"`macval(`opt')'"'=="" & `"`no`opt''"'=="" {
					local line: list retok line
					local `opt' `"`macval(line)'"'
				}
			}
			file close `file'
		}
	}

*title option
	if `"`macval(prehead)'"'=="" & `"`macval(posthead)'"'=="" ///
	 & `"`macval(prefoot)'"'=="" & `"`macval(postfoot)'"'=="" ///
	 & `"`macval(title)'"'!="" {
		local prehead `"`"`macval(title)'"'"'
	}

*Generate/clean-up cell contents
	if `"`:list clean cells'"'=="" {
		local cells b
	}
	else if `"`:list clean cells'"'=="none" {
		local cells
	}
	CellsCheck `"`cells'"'

*Special treatment of confidence intervalls
	if "`level'"=="" local level $S_level
	if `level'<10 | `level'>99 {
		di as error "level(`level') invalid"
		exit 198
	}
	if `: list posof "ci" in values' {
		if `"`macval(ci_label)'"'=="" {
			local ci_label "ci`level'"
		}
		if `"`macval(ci_par)'"'=="" {
			if "`ci_separate'"=="" local ci_par `""" , """'
		}
		tokenize `"`macval(ci_par)'"'
		local ci_l_par `""`macval(1)'" "`macval(2)'""'
		local ci_u_par `""" "`macval(3)'""'
	}
	if `: list posof "ci_l" in values' & `"`macval(ci_l_label)'"'=="" {
		local ci_l_label "min`level'"
	}
	if `: list posof "ci_u" in values' & `"`macval(ci_u_label)'"'=="" {
		local ci_u_label "max`level'"
	}

*Formats
	if "`b_fmt'"=="" local b_fmt %9.0g
	foreach v of local values {
		if "``v'_fmt'"=="" local `v'_fmt "`b_fmt'"
		if `"`macval(`v'_label)'"'=="" {
			local `v'_label "`v'"
		}
	}

*Check margin option / prepare discrete option
	if "`margin'"!="" {
		if !inlist("`margin'","margin","u","c","p") {
			di as error "margin(`margin') invalid"
			exit 198
		}
		if `"`macval(discrete)'"'!="" {
			gettoken discrete discrete2: discrete
		}
	}
	else local discrete

*Formats/labels/stars for statistics
	local stats: list uniq stats
	if "`statsfmt'"=="" local statsfmt: word 1 of `b_fmt'
	if `"`statslabels'"'=="" local statslabels `"`stats'"'
	local p: list posof "p" in stats
	if "`statsstar'"!="" | `p' local p "p df_m F chi2"
	else local p

*Starposition
	foreach v of local values {
		if "``v'_star'"!="" {
			Star2Cells `"`cells'"' "`v'"
		}
	}

*Check/define starlevels/make levelslegend
	if index(`"`cells'"',"star") | `"`statsstar'"'!="" {
		if `"`macval(starlevels)'"'=="" ///
		 local starlevels "* 0.05 ** 0.01 *** 0.001"
		CheckStarvals `"`macval(starlevels)'"'
	}

*Modelwidth/varwidth/starwidth
	if "`modelwidth'"=="" local modelwidth 0
	if "`varwidth'"=="" local varwidth 0
	local starwidth 0
	if `modelwidth'>0 {
		local fmt_m "%`modelwidth's"
		if index(`"`cells'"',"star") | `"`statsstar'"'!="" {
			Starwidth `"`macval(starlevels)'"'
			local starwidth `value'
			local fmt_star "%-`value's"
		}
	}
	if `varwidth'>0 local fmt_v "%-`varwidth's"

*Get coefficients/variances/statistics: est_table
	qui estimates table `anything', ///
	 stats(df_r `p' `stats') `equations' `keep' `drop'
	local models `r(names)'
	tempname B D St
	mat `St'=r(stats)
	mat `B'=r(coef)
	local R=rowsof(`B')
	local varlist: rownames `B'
	local eqlist: roweq `B', q
	local eqlist: list clean eqlist
	local eqs: list uniq eqlist
	forv i = 1/`R' {
		local temp: word `i' of `eqlist'
		if `"`temp'"'=="_" local temp
		else if `"`temp'"'!="" {
			local temp `"`temp':"'
		}
		local fullvarlist `"`fullvarlist'`"`temp'`:word `i' of `varlist''"' "'
	}
	local fullvarlist: list clean fullvarlist
	mat `D' = J(`R',1,0)
	mat rown `D' = `fullvarlist'

*Calculate p-value
	if "`p'"!="" {
		forv m = 1/`:word count `models'' {
			if `St'[2,`m']==.z {
				if `St'[4,`m']<.z {
					mat `St'[2,`m'] = Ftail(`St'[3,`m'],`St'[1,`m'],`St'[4,`m'])
				}
				else if `St'[5,`m']<.z {
					mat `St'[2,`m'] = chi2tail(`St'[3,`m'],`St'[5,`m'])
				}
			}
		}
	}

*Make dictionary of matched equations
	if `"`equations'"'!="" {
		ParseEquations, eqs(`eqs') m(`: word count `models'') `equations'
	}

*Prepare keep/drop
	foreach v of local values {
		if `"``v'_keep'"'!="" {
			if `"``v'_drop'"'!="" {
				di as error "`v'(drop()) not allowed if `v'(keep()) is specified"
				exit 198
			}
			Keep `B' `"``v'_keep'"'
		}
		else if `"``v'_drop'"'!="" {
			Drop `B' `"``v'_drop'"'
		}
		else local temp `"`fullvarlist'"'
		local `v'_drop: list fullvarlist - temp
	}

*Prepare unstack
	if "`unstack'"!="" {
		local varlist: list uniq varlist
		local cons _cons
		if `:list cons in varlist' {
			local varlist: list varlist - cons
			local varlist: list varlist | cons
		}
		local R: word count `varlist'
		local eqswide `"`eqs'"'
	}
	else local eqswide "_"

*RestoreEstimates: mlabelsdepvars, marginals, cell stats, overall stats
	if index(`"`cells'"',"star") & !`: list posof "p" in values' ///
	 local values "`values' p"
	local values: subinstr local values "ci" "ci_l ci_u", word
	if "`eform'"=="eform" {
		local eform 1
		forv i = 2/`: word count `models'' {
			local eform `eform' 1
		}
	}
	foreach v of local values {
		tempname _`v'
		mat `_`v''=J(rowsof(`B'),`:list sizeof models',.z)
		mat rown `_`v'' = `fullvarlist'
		mat coln `_`v'' = `models'
		if !inlist("`v'","b","se","t","p","ci_l","ci_u") {
			local xvalues: list xvalues | v
			local x_v: list x_v | _`v'
		}
	}
	if "`unstack'"!="" {
		if "`p'"!="" local p p
		local p: list stats | p
		local aicbicrank "aic bic rank"
		local p: list p - aicbicrank
	}
	else local p
	if "`p'"!="" {
		local value
		foreach eq of local eqswide {
			foreach v of local p {
				local value `"`value'`"`eq':`v'"' "'
			}
		}
		tempname mestats
		mat `mestats' = J(`:list sizeof value',`:list sizeof models',.z)
		mat rown `mestats' = `value'
		mat coln `mestats' = `models'
	}
	if "`mlabelsdepvars'"!="" | "`margin'"!="" | "`xvalues'"!="" ///
	 | "`p'"!="" | "`label'"!="" {
		RestoreEstimates `B' `D' "`models'" "`eqnames'" `"`eqspec'"' ///
		 "`mlabelsdepvars'" `"`macval(mlabels)'"' "`label'" "`margin'" ///
		 `"`meqs'"' "`xvalues'" "`x_v'" "`St'" "`p'" "`mestats'"
	}
	foreach v of local values {
		if "`v'"=="b" {
			_estout_b `B' `_b' "`eform'"
		}
		else if "`v'"=="se" {
			_estout_se `B' `_se' "`eform'"
		}
		else if "`v'"=="t" {
			_estout_t `B' `_t' `t_abs'
		}
		else if "`v'"=="p" {
			_estout_p `B' `_p' `St'
		}
		else if "`v'"=="ci_l" {
			_estout_ci_l `B' `_ci_l' `St' `level' "`eform'"
		}
		else if "`v'"=="ci_u" {
			_estout_ci_u `B' `_ci_u' `St' `level' "`eform'"
		}
	}

*Model labels
	if `"`macval(mlabels)'"'=="" {
		local mlabels "`models'"
	}
	if "`mlabelsnumbers'"!="" {
		NumberMlabels `: word count `models'' `"`macval(mlabels)'"'
	}

*Equations labels
	if `"`macval(eqlabels)'"'=="" {
		if "`label'"!="" {
			foreach eq of local eqs {
				local temp
				capture confirm variable `eq'
				if !_rc {
					local temp: var l `eq'
				}
				if `"`temp'"'=="" local temp "`eq'"
				local eqlabels `"`eqlabels'`"`temp'"' "'
			}
		}
		else local eqlabels `"`eqs'"'
	}

*Column labels
	if `"`macval(collabels)'"'=="" {
		forv j = 1/`ncols' {
			local temp
			forv i = 1/`nrows' {
				local v: word `i' of `cells'
				local v: word `j' of `v'
				ParseV `v'
				if "`v'"!="" {
					if `"`macval(temp)'"'!="" {
						local temp `"`macval(temp)'/"'
					}
					local temp `"`macval(temp)'`macval(`v'_label)'"'
				}
			}
			local collabels `"`macval(collabels)'`"`macval(temp)'"' "'
		}
	}

*Determine table layout
	local m 1
	local starcol 0
	foreach model of local models {
		local e 0
		foreach eq of local eqswide {
			if `"`statsstar'"'!="" local starcol 1
			local ++e
			if "`unstack'"!="" {
				ModelEqCheck `B' "`eq'" `m'
				if !`value' continue
			}
			local eqsrow "`eqsrow'`e' "
			local modelsrow "`modelsrow'`m' "
			local k 0
			local something 0
			forv j = 1/`ncols' {
				local col
				forv i = 1/`nrows' {
					local row: word `i' of `cells'
					local v: word `j' of `row'
					ParseV `v'
					if `istar' local starcol 1
					if "`v'"=="" local v "."
					else {
						if "`:word `m' of ``v'_pattern''"=="0" local v ".`v'"
					}
					local v "`v'`vstar'"
					local col "`col'`v' "
				}
				local nocol 1
				foreach v of local col {
					if index("`v'",".")!=1 {
						local nocol 0
						continue, break
					}
				}
				if !`nocol' {
					local colsrow "`colsrow'`j' "
					if `++k'>1 {
						local modelsrow "`modelsrow'`m' "
						local eqsrow "`eqsrow'`e' "
					}
					local starsrow "`starsrow'`starcol' "
					local starcol 0
					Add2Vblock `"`vblock'"' "`col'"
					local something 1
				}
			}
			if !`something' {
				local col
				forv i = 1/`nrows' {
					local col "`col'. "
				}
				Add2Vblock `"`vblock'"' "`col'"
				local colsrow "`colsrow'1 "
				local starsrow "`starsrow'`starcol' "
			}
		}
	local ++m
	}
	VblockCheck `"`vblock'"'
	local ncols = `: word count `starsrow'' + 1
	if "`stardetach'"!="" {
		foreach i of local starsrow {
			local ncols = `ncols'+`i'
		}
	}

*Open temp. output file / check begin, delimiter, end, width of delimiter
	tempfile tfile
	tempname file
	file open `file' using `"`tfile'"', write text
	foreach opt in begin delimiter end {
		capture file write `file' `macval(`opt')'
		if _rc {
			local `opt' `"`"`macval(`opt')'"'"'
		}
	}
	file close `file'
	if "`mgroupsspan'"!="" | "`mlabelsspan'"!="" ///
	 | "`eqlabelsspan'"!="" | "`collabelsspan'"!="" {
		if `modelwidth'>0 {
			file open `file' using `"`tfile'"', write text replace
			file write `file' `macval(delimiter)'
			file close `file'
			file open `file' using `"`tfile'"', read text
			file read `file' delwidth
			file close `file'
			local delwidth = length(`"`macval(delwidth)'"')
		}
		else local delwidth 0
	}
	if "`stardetach'"!="" {
		local stardetach `"`macval(delimiter)'"'
	}
	file open `file' using `"`tfile'"', write text replace

*Write prehead
	foreach line of local prehead {
		InsertAtVariables `"`macval(line)'"' "`ncols'" 0 ///
		 "`:word count `models''" `"`macval(title)'"' ///
		 `"`macval(discrete)'`macval(discrete2)'"' `"`macval(starlegend)'"'
		file write `file' `"`macval(value)'"' _n
	}

*Write head: Models groups
	if "`mgroupsnone'"=="" & `"`macval(mgroups)'"'!="" {
		MgroupsPattern "`modelsrow'" "`mgroupspattern'"
		Abbrev `varwidth' `"`macval(mgroupslhs)'"' "`abbrev'"
		file write `file' `"`macval(mgroupsbegin)'"' `macval(begin)' ///
		 `fmt_v' (`"`macval(value)'"')
		WriteCaption `"`file'"' `"`macval(delimiter)'"' ///
		 `"`macval(stardetach)'"' "`mgroupspattern'" "`mgroupspattern'" ///
		 `"`macval(mgroups)'"' "`starsrow'" "`mgroupsspan'" "`abbrev'" ///
		 "`modelwidth'" "`fmt_m'" "`delwidth'" "`starwidth'" ///
		 `"`macval(mgroupserepeat)'"' `"`macval(mgroupsprefix)'"' ///
		 `"`macval(mgroupssuffix)'"'
		file write `file' `macval(end)' ///
		 `"`macval(mgroupsend)'`macval(value)'"' _n
	}

*Write head: Models captions
	if "`mlabelsnone'"=="" {
		Abbrev `varwidth' `"`macval(mlabelslhs)'"' "`abbrev'"
		file write `file' `"`macval(mlabelsbegin)'"' `macval(begin)' ///
		 `fmt_v' (`"`macval(value)'"')
		WriteCaption `"`file'"' `"`macval(delimiter)'"' ///
		 `"`macval(stardetach)'"' "`modelsrow'" "`modelsrow'"  ///
		 `"`macval(mlabels)'"' "`starsrow'" "`mlabelsspan'" "`abbrev'"  ///
		 "`modelwidth'" "`fmt_m'" "`delwidth'" "`starwidth'" ///
		 `"`macval(mlabelserepeat)'"' `"`macval(mlabelsprefix)'"' ///
		 `"`macval(mlabelssuffix)'"'
		file write `file' `macval(end)' ///
		 `"`macval(mlabelsend)'`macval(value)'"' _n
	}

*Write head: Equations captions
	if `"`eqswide'"'!="_" & "`eqlabelsnone'"=="" {
		Abbrev `varwidth' `"`macval(eqlabelslhs)'"' "`abbrev'"
		file write `file' `"`macval(eqlabelsbegin)'"' `macval(begin)' ///
		 `fmt_v' (`"`macval(value)'"')
		WriteCaption `"`file'"' `"`macval(delimiter)'"' ///
		 `"`macval(stardetach)'"' "`eqsrow'" "`modelsrow'" ///
		 `"`macval(eqlabels)'"' "`starsrow'" "`eqlabelsspan'"  "`abbrev'" ///
		 "`modelwidth'" "`fmt_m'" "`delwidth'" "`starwidth'" ///
		 `"`macval(eqlabelserepeat)'"' `"`macval(eqlabelsprefix)'"' ///
		 `"`macval(eqlabelssuffix)'"'
		file write `file' `macval(end)' ///
		 `"`macval(eqlabelsend)'`macval(value)'"' _n
	}

*Write head: Columns captions
	if `"`macval(collabels)'"'!="" & "`collabelsnone'"=="" {
		Abbrev `varwidth' `"`macval(collabelslhs)'"' "`abbrev'"
		file write `file' `"`macval(collabelsbegin)'"' `macval(begin)' ///
		 `fmt_v' (`"`macval(value)'"')
		WriteCaption `"`file'"' `"`macval(delimiter)'"' ///
		 `"`macval(stardetach)'"' "`colsrow'" "" `"`macval(collabels)'"' ///
		 "`starsrow'" "`collabelsspan'" "`abbrev'" "`modelwidth'" "`fmt_m'" ///
		 "`delwidth'" "`starwidth'" `"`macval(collabelserepeat)'"' ///
		 `"`macval(collabelsprefix)'"' `"`macval(collabelssuffix)'"'
		file write `file' `macval(end)' ///
		 `"`macval(collabelsend)'`macval(value)'"' _n
	}

*Write posthead
	foreach line of local posthead {
		InsertAtVariables `"`macval(line)'"' "`ncols'" 0 ///
		 "`:word count `models''" `"`macval(title)'"' ///
		 `"`macval(discrete)'`macval(discrete2)'"' `"`macval(starlegend)'"'
		file write `file' `"`macval(value)'"' _n
	}

*Write body of table
*Loop over table rows
	tempname first
	if `"`vblock'"'!="" {
		local e 0
		forv r = 1/`R' {

*Write equation name/label
			if "`unstack'"=="" {
				local eqvar: word `r' of `fullvarlist'
				if `"`eqs'"'!="_" {
					local eqrlast `"`eqr'"'
					local eqr: word `r' of `eqlist'
					if `"`eqr'"'!=`"`eqrlast'"' & "`eqlabelsnone'"=="" {
					local value: word `++e' of `macval(eqlabels)'
					file write `file' `"`macval(eqlabelsbegin)'"' `macval(begin)'
					WriteEqrow `"`file'"' `"`macval(delimiter)'"' ///
					 `"`macval(stardetach)'"' `"`macval(value)'"' "`starsrow'" ///
					 "`eqlabelsspan'" "`varwidth'" "`fmt_v'" "`abbrev'" ///
					 "`modelwidth'" "`fmt_m'" "`delwidth'" "`starwidth'" ///
					 `"`macval(eqlabelsprefix)'"' `"`macval(eqlabelssuffix)'"'
					file write `file' `macval(end)' `"`macval(eqlabelsend)'"' _n
					}
				}
			}

*Write variable name/label
			local var: word `r' of `varlist'
			if "`varlabelsnone'"=="" {
				VarInList "`var'" "`unstack'" `"`eqvar'"' ///
				 `"`eqr'"' `"`macval(varlabelsblist)'"'
				if `"`macval(value)'"'!="" {
					InsertAtVariables `"`macval(value)'"' "`ncols'" 1
					file write `file' `"`macval(value)'"'
				}
				if "`label'"!="" {
					local temp = index("`var'",".")
					capture local varl: var l `=substr("`var'",`temp'+1,.)'
					if _rc | `"`varl'"'=="" {
						local varl `=substr("`var'",`temp'+1,.)'
					}
					local varl `"`=substr("`var'",1,`temp')'`macval(varl)'"'
				}
				else local varl `var'
				VarInList "`var'" "`unstack'" `"`eqvar'"' ///
				 `"`eqr'"' `"`macval(varlabels)'"'
				if `"`macval(value)'"'!="" {
					local varl `"`macval(value)'"'
				}
				if `"`macval(discrete)'"'!="" {
					local temp 0
					if "`unstack'"=="" {
						if `D'[`r',1]==1 local temp 1
					}
					else {
						foreach eqr of local eqswide {
							if `D'[rownumb(`D',`"`eqr':`var'"'),1]==1 local temp 1
						}
					}
					if `temp'==1 & `temp'<. {
						local varl `"`macval(varl)'`macval(discrete)'"'
					}
				}
			}
			else local varl
			Abbrev `varwidth' ///
			 `"`macval(varlabelsprefix)'`macval(varl)'`macval(varlabelssuffix)'"' ///
			 "`abbrev'"
			file write `file' `"`macval(varlabelsbegin)'"' `macval(begin)' ///
			 `fmt_v' (`"`macval(value)'"')

*Write table cells
			local newrow 0
			mat `first'=J(1,`:word count `models'',1)
			foreach row of local vblock {
				local c 0
				local skiprow 1
				foreach v of local row {
					local ++c
					ParseV `v'
					if index("`v'",".")==1 continue
					if "`unstack'"!="" {
						local eqr: word `:word `c' of `eqsrow'' of `eqs'
						if `"`eqr'"'!="_" local eqvar `"`eqr':`var'"'
						else local eqvar "`var'"
					}
					if rownumb(`B',`"`eqvar'"')<. {
						local temp=substr("`v'",index("`v'",".")+1,.)
						if `: list eqvar in `temp'_drop' continue
						local skiprow 0
						continue, break
					}
				}
				if `skiprow' continue
				local c 0
				foreach v of local row {
					ParseV `v'
					local m: word `++c' of `modelsrow'
					if "`unstack'"!="" {
						local eqr: word `:word `c' of `eqsrow'' of `eqs'
						local rr=rownumb(`B',`"`eqr':`var'"')
						if `"`eqr'"'!="_" local eqvar `"`eqr':`var'"'
						else local eqvar "`var'"
						if `rr'>=. local v ".`v'"
					}
					else local rr `r'
					if index("`v'",".")!=1 {
						local temp = substr("`v'",index("`v'",".")+1,.)
						if `: list eqvar in `temp'_drop' local v ".`temp'"
					}
					if `newrow' & `c'==1 {
						file write `file' `macval(end)' _n `macval(begin)' _skip(`varwidth')
					}
					if index("`v'",".")==1 local value
					else if `B'[`rr',`m'*2-1]==0 & `B'[`rr',`m'*2]==0 & "`margin'"=="" {
						if `first'[1,`m'] {
							local value "(dropped)"
							mat `first'[1,`m']=0
						}
						else local value
					}
					else if "`v'"=="ci" {
						if `_ci_l'[`rr',`m']==.z local value
						else if `B'[`rr',`m'*2-1]<. & `_ci_l'[`rr',`m']>=. local value .
						else if `_ci_l'[`rr',`m']<. {
							local format: word `r' of `ci_fmt'
							if "`format'"=="" local format: word `:word count `ci_fmt'' of `ci_fmt'
							local value = `_ci_l'[`rr',`m']
							vFormat `value' `format' "`lz'" `"`macval(dmarker)'"' ///
							 `"`macval(msign)'"' `"`macval(ci_l_par)'"'
							local temp "`macval(value)'"
							local value = `_ci_u'[`rr',`m']
							vFormat `value' `format' "`lz'" `"`macval(dmarker)'"' ///
							 `"`macval(msign)'"' `"`macval(ci_u_par)'"'
							local value `"`macval(temp)'`macval(value)'"'
						}
					}
					else if `_`v''[`rr',`m']==.z local value
					else if `_`v''[`rr',`m']>=.  {
						local value .
					}
					else {
						local format: word `r' of ``v'_fmt'
						if "`format'"=="" local format: word `:word count ``v'_fmt'' of ``v'_fmt'
						local value = `_`v''[`rr',`m']
						vFormat `value' `format' "`lz'" `"`macval(dmarker)'"' ///
						 `"`macval(msign)'"' `"`macval(`v'_par)'"'
					}
					file write `file' `macval(delimiter)' `fmt_m' (`"`macval(value)'"')
					if `istar' & index("`v'",".")!=1 {
						Stars `"`macval(starlevels)'"' `_p'[`rr',`m']
						file write `file' `macval(stardetach)' `fmt_star' (`"`macval(value)'"')
					}
					else if `:word `c' of `starsrow''==1 {
						file write `file' `macval(stardetach)' _skip(`starwidth')
					}
				}
				local newrow 1
			}

*End of table row
			if `r'==`R' & "`varlabelslast'"=="" local varlabelsend
			VarInList "`var'" "`unstack'" `"`eqvar'"' `"`eqr'"' ///
			 `"`macval(varlabelselist)'"'
			InsertAtVariables `"`macval(value)'"' "`ncols'" 1
			file write `file' `macval(end)' ///
			 `"`macval(varlabelsend)'`macval(value)'"' _n
		}
	}

*Write prefoot
	foreach line of local prefoot {
		InsertAtVariables `"`macval(line)'"' "`ncols'" 0 ///
		 "`:word count `models''" `"`macval(title)'"' ///
		 `"`macval(discrete)'`macval(discrete2)'"' `"`macval(starlegend)'"'
		file write `file' `"`macval(value)'"' _n
	}

*Write foot of table (statistics)
	local S: word count `stats'
	local eqr "_"
	forv r = 1/`S' {
		local stat: word `r' of `macval(statslabels)'
		if `"`stat'"'=="" local stat: word `r' of `stats'
		if "`statslabelsnone'"!="" local stat
		Abbrev `varwidth' ///
		 `"`macval(statslabelsprefix)'`macval(stat)'`macval(statslabelssuffix)'"' ///
		 "`abbrev'"
		file write `file' `"`macval(statslabelsbegin)'"' `macval(begin)' ///
		 `fmt_v' (`"`macval(value)'"')
		local temp: word `r' of `statsfmt'
		if "`temp'"!="" local format "`temp'"
		local stat: word `r' of `stats'
		local lastm
		local lasteq
		local c 0
		foreach m of local modelsrow {
			local ++c
			local value
			local eq: word `:word `c' of `eqsrow'' of `eqs'
			if "`m'"!="`lastm'" | `"`eq'"'!="`lasteq'" {
				if "`m'"!="`lastm'" {
					local usemestats 0
					local rr = rownumb(`St',`"`stat'"')
					if `St'[`rr',`m']!=.z {
						local value = `St'[`rr',`m']
					}
					else {
						cap confirm matrix `mestats'
						if !_rc local usemestats 1
					}
				}
				if `usemestats' {
					local rr = rownumb(`mestats',`"`eq':`stat'"')
					if `mestats'[`rr',`m']!=.z {
						local value = `mestats'[`rr',`m']
					}
				}
				if "`value'"!="" {
					vFormat `value' "`format'" "`lz'" `"`macval(dmarker)'"' ///
					 `"`macval(msign)'"'
				}
			}
			file write `file' `macval(delimiter)' `fmt_m' (`"`macval(value)'"')
			if `"`macval(value)'"'!="" & `:list stat in statsstar' {
				if `usemestats' {
					local rr=rownumb(`mestats',`"`eq':p"')
					Stars `"`macval(starlevels)'"' `mestats'[`rr',`m']
				}
				else {
					Stars `"`macval(starlevels)'"' `St'[2,`m']
				}
				file write `file' `macval(stardetach)' `fmt_star' (`"`macval(value)'"')
			}
			else if `:word `c' of `starsrow''==1 {
				file write `file' `macval(stardetach)' _skip(`starwidth')
			}
			local lastm "`m'"
			local lasteq `"`eq'"'
		}
		if `r'==`S' & "`statslabelslast'"=="" local statslabelsend
		file write `file' `macval(end)' `"`macval(statslabelsend)'"' _n
	}

*Write postfoot
	local discrete: list retok discrete
	foreach line of local postfoot {
		InsertAtVariables `"`macval(line)'"' "`ncols'" 0 ///
		 "`:word count `models''" `"`macval(title)'"' ///
		 `"`macval(discrete)'`macval(discrete2)'"' `"`macval(starlegend)'"'
		file write `file' `"`macval(value)'"' _n
	}

*Write legend (starlevels, marginals)
	if "`legend'"!="" {
		if `"`macval(discrete2)'"'!="" {
			mat `D' = `D''*`D'
			if `D'[1,1]!=0 {
				file write `file' `"`macval(discrete)'`macval(discrete2)'"' _n
			}
		}
		if `"`macval(starlegend)'"'!="" {
			file write `file' `"`macval(starlegend)'"' _n
		}
	}

*Finish: copy tempfile to user file / type to screen
	file close `file'
	local S: word count `macval(substitute)'
	if `"`using'"'!="" {
		tempname file2
		file open `file2' `using', write text `replace' `append'
	}
	file open `file' using `"`tfile'"', read text
	file read `file' temp
	if "`type'"!="" di
	while r(eof)==0 {
		forv s = 1(2)`S' {
			local from: word `s' of `macval(substitute)'
			local to:  word `=`s'+1' of `macval(substitute)'
			local temp: subinstr local temp `"`macval(from)'"' `"`macval(to)'"', all
		}
		if `"`using'"'!="" {
			file write `file2' `"`macval(temp)'"' _n
		}
		if "`type'"!="" {
			if "`showtabs'"!="" {
				local temp: subinstr local temp "`=char(9)'" "<T>", all
			}
			di as res _asis `"`macval(temp)'"'
		}
		file read `file' temp
	}
	file close `file'
	if `"`using'"'!="" {
		file close `file2'
	}

end

program ParseValueSubopts
	syntax anything [ , NOStar Star fmt(string) Label(string) ///
	 NOPAR par PAR2(string asis) Keep(string) Drop(string) PATtern(string) NOABS abs ]
	CheckPattern `"`pattern'"' "`anything'"
	if `"`macval(par2)'"'!="" {
		local par `"`macval(par2)'"'
	}
	else if "`par'"!="" {
		if "`anything'"=="ci" local par "[ , ]"
		else if "`anything'"=="ci_l" local par `"[ """'
		else if "`anything'"=="ci_u" local par `""" ]"'
		else local par "( )"
	}
	foreach opt in star par abs {
		if "`no`opt''"!="" c_local no`anything'_`opt' 1
		else c_local `anything'_`opt' `"`macval(`opt')'"'
	}
	foreach opt in fmt label keep drop pattern {
		c_local `anything'_`opt' `"`macval(`opt')'"'
	}
end

program CheckPattern
	args pattern option
	foreach p of local pattern {
		if !( "`p'"=="1" | "`p'"=="0" ) {
			di as error `""`pattern'" invalid in `option'(... pattern())"'
			exit 198
		}
	}
end

program ParseStatsSubopts
	syntax [anything] [ , fmt(string) Labels(string asis) ///
	 NOStar Star Star2(string) ]
	foreach opt in fmt labels {
		c_local stats`opt' `"`macval(`opt')'"'
	}
	if "`nostar'"!="" c_local nostatsstar 1
	else if "`star2'"!="" {
		local anything: list anything | star2
		c_local statsstar "`star2'"
	}
	else if "`star'"!="" {
		local star2: word 1 of `anything'
		c_local statsstar "`star2'"
	}
	c_local stats "`anything'"
	c_local stats2
end

program ParseLabelsSubopts
	gettoken type 0: 0
	syntax [anything] [ , NONUMbers NUMbers NODEPvars DEPvars NONONE NONE ///
	 NOSPAN span Prefix(string) Suffix(string) Begin(string) ///
	 End(string) BList(string asis) EList(string asis) ///
	 ERepeat(string) NOLast Last lhs(string) PATtern(string) ]
	CheckPattern `"`pattern'"' "`type'"
	foreach opt in prefix suffix begin end blist elist erepeat lhs pattern {
		c_local `type'`opt' `"`macval(`opt')'"'
	}
	foreach opt in numbers depvars span none last {
		if "`no`opt''"!="" c_local no`type'`opt' 1
		else c_local `type'`opt' "``opt''"
	}
	c_local `type' `"`macval(anything)'"'
end

program ReadLine
	args max file
	local end 0
	file read `file' temp1
	while r(eof)==0 {
		local j 1
		local temp2
		local temp3: piece `j++' `max' of `"`macval(temp1)'"'
		if `"`temp3'"'=="" | index(`"`temp3'"',"*")==1 ///
		 | index(`"`temp3'"',"//")==1 {
			file read `file' temp1
			continue
		}
		while `"`temp3'"'!="" {
			local comment=index(`"`macval(temp3)'"'," ///")
			if `comment' {
				local temp3=substr(`"`macval(temp3)'"',1,`comment')
				local temp2 `"`macval(temp2)'`macval(temp3)'"'
				local end 0
				continue, break
			}
			local comment=index(`"`macval(temp3)'"'," //")
			if `comment' {
				local temp3=substr(`"`macval(temp3)'"',1,`comment')
				local temp2 `"`macval(temp2)'`macval(temp3)'"'
				local end 1
				continue, break
			}
			local temp2 `"`macval(temp2)'`macval(temp3)'"'
			local temp3: piece `j++' `max' of `"`macval(temp1)'"'
			local end 1
		}
		if `end' {
			local line `"`macval(line)'`macval(temp2)'"'
			continue, break
		}
		else {
			local line `"`macval(line)'`macval(temp2)'"'
			file read `file' temp1
		}
	}
	c_local line `"`macval(line)'"'
end

program CellsCheck
	args cells
	local ncols 0
	local nrows 0
	foreach row of local cells {
		local newrow
		foreach col of local row {
			if !`:list col in values' {
				local newrow: list newrow | col
				local values: list values | col
			}
		}
		if "`newrow'"!="" {
			local ncols = max(`ncols',`:list sizeof newrow')
			local newcells `"`newcells'"`newrow'" "'
			local ++nrows
		}
	}
	local newcells: list retok newcells
	c_local cells `"`newcells'"'
	c_local ncols `ncols'
	c_local nrows `nrows'
	c_local values `values'
end

program Star2Cells
	args cells star
	local newcells
	foreach row of local cells {
		local newrow
		foreach col of local row {
			if "`col'"=="`star'" {
				local col "`col'star"
			}
			local newrow: list newrow | col
		}
		local newcells `"`newcells'"`newrow'" "'
	}
	local newcells: list retok newcells
	c_local cells `"`newcells'"'
end

program CheckStarvals
	args starlevels
	local nstar: word count `macval(starlevels)'
	local nstar = `nstar'/2
	capture confirm integer number `nstar'
	if _rc {
		di as error "unmatched list of significance symbols and levels"
		exit 198
	}
	local istar 1
	forv i = 1/`nstar' {
		local iistar: word `=`i'*2' of `macval(starlevels)'
		confirm number `iistar'
		if `iistar'>`istar' | `iistar'<=0 {
			di as error "significance levels out of order or out of range (0,1]"
			exit 198
		}
		local istar `iistar'
		local isym: word `=`i'*2-1' of `macval(starlevels)'
		if `"`macval(legend)'"'!="" {
			local legend `"`macval(legend)', "'
		}
		local legend `"`macval(legend)'`macval(isym)' p<`istar'"'
	}
	c_local starlegend `"`macval(legend)'"'
end

program Starwidth
	args starlevels
	local nstar: word count `macval(starlevels)'
	forv i = 2(2)`nstar' {
		local istar: word `=`i'-1' of `macval(starlevels)'
		local width = max(length("`width'"),length(`"`macval(istar)'"'))
	}
	c_local value `width'
end

program ParseV
	args v
	local istar = index("`v'","star")
	if `istar' {
		local v = substr("`v'",1,`istar'-1)
		local vstar star
	}
	c_local v `v'
	c_local vstar `vstar'
	c_local istar `istar'
end

program ParseEquations
	syntax, eqs(string asis) m(numlist int max=1 >0) EQuations(string)
	local eqspec  : subinstr local equations ":" " ", all
	local eqspec0 : subinstr local eqspec "#" "" , all
	local iterm 0
	gettoken term eqspec : eqspec0 , parse(",")
	local eqspec0
	while "`term'" != "" {
		local ++iterm
		gettoken eqname oprest: term, parse("=")
		gettoken op rest : oprest, parse("=")
		if trim(`"`op'"') != "=" {
			local op "="
			local rest "`eqname'"
			local eqname #`iterm'
		}
		if `:list eqname in eqs' {
			if `:list sizeof rest'==1 {
				local temp `rest'
				forv i = 2/`m' {
					local rest `rest' `temp'
				}
			}
			local eqnames `eqnames' `eqname'
			local eqspec0 `"`eqspec0'"`rest'" "'
		}
		if "`eqspec'" == "" {
			continue, break
		}
		gettoken term eqspec: eqspec , parse(",")
		assert "`term'" == ","
		gettoken term eqspec: eqspec , parse(",")
	}
	local eqspec0: list retok eqspec0
	c_local eqspec   `"`eqspec0'"'
	c_local eqnames  `eqnames'
end

program RestoreEstimates
	args B D models eqnames eqspec mldepv labels label margin meqs ///
	 xv x_v St stats mestats
	local elab = "`mldepv'"=="" & `"`macval(labels)'"'=="" & "`label'"!=""
	tempname hcurrent
	_est hold `hcurrent', restore nullok estsystem
	nobreak {
		local j 1
		foreach m of local models {
			if "`m'"=="." _est unhold `hcurrent'
			else {
				_est unhold `m'
				capt confirm new var _est_`m'
				if _rc drop _est_`m'
			} //estimates restore `m', noh drop
			MatchEqlist "`B'" `j' "`eqnames'" `"`eqspec'"'
			if "`mldepv'"!="" {
				local var: word 1 of `e(depvar)'
				if "`label'"!="" {
					local temp = index("`var'",".")
					capture local lab: var l `=substr("`var'",`temp'+1,.)'
					if _rc | `"`lab'"'=="" {
						local lab `=substr("`var'",`temp'+1,.)'
					}
					local lab `"`=substr("`var'",1,`temp')'`macval(lab)'"'
				}
				else local lab "`var'"
				local labels `"`labels'`"`lab'"' "'
			}
			if `elab' {
				local lab `"`e(_estimates_title)'"'
				if `"`lab'"'=="" local lab "`m'"
				local labels `"`labels'`"`lab'"' "'
			}
			if "`margin'"!="" GetMarginals  `B' `D' `j' "`margin'" `"`meqs'"'
			if "`xv'"!="" {
				local x 0
				foreach v of local xv {
					local ++x
					local _v: word `x' of `x_v'
					local options: word `x' of `macval(xoptions)'
					GetValues `v' `j' `B' `_v' `"`eqmatch'"'
				}
			}
			if ( e(cmd)=="reg3" | e(cmd)=="sureg" | e(cmd)=="mvreg" ) ///
			 & "`stats'"!="" {
				GetStats `j' `mestats' `"`eqmatch'"' "`stats'"
			}
			if "`m'"=="." _est hold `hcurrent', restore nullok estsystem
			else _est hold `m', nullok estimates varname(_est_`m') //estimates store `m', title(`"`e(_estimates_title)'"')
			local ++j
		}
		_est unhold `hcurrent'
	}
	c_local mlabels `"`macval(labels)'"'
end

program MatchEqlist
	args B j eqnames eqspec
	local eqs1: coleq e(b), q
	local eqs1: list uniq eqs1
	local eqs1: list clean eqs1
	local rest `"`eqs1'"'
	local eqs2: roweq `B', q
	local eqs2: list uniq eqs2
	local eqs2: list clean eqs2
	local i 0
	foreach eq of local eqs2 {
		if `:list eq in eqnames' {
			local ieq: word `++i' of `eqspec'
			local ieq: word `j' of `ieq'
			if `ieq'<. {
				local ieq: word `ieq' of `eqs1'
				if `:list ieq in rest' {
					local eqmatch `"`eqmatch'`"`ieq' `eq'"' "'
					local rest: list rest - ieq
				}
			}
		}
		else if `:list eq in rest' {
			local eqmatch `"`eqmatch'`"`eq' `eq'"' "'
			local rest: list rest - eq
		}
	}
	local eqmatch: list clean eqmatch
	c_local eqmatch `"`eqmatch'"'
end

program GetMarginals
	args B D j margin meqs
		tempname dfdx
		local type `e(Xmfx_type)'
		if "`type'"!="" {
			mat `dfdx' = e(Xmfx_`type')
			capture confirm matrix e(Xmfx_se_`type')
			if _rc==0 {
				mat `dfdx' = `dfdx' \ e(Xmfx_se_`type')
			}
			if "`e(Xmfx_discrete)'"=="discrete" local dummy `e(Xmfx_dummy)'
		}
		else if "`e(cmd)'"=="dprobit" {
			mat `dfdx' = e(dfdx) \ e(se_dfdx)
			local dummy `e(dummy)'
		}
		else if "`e(cmd)'"=="tobit" & inlist("`margin'","u","c","p") {
			capture confirm matrix e(dfdx_`margin')
			if _rc==0 {
				mat `dfdx' = e(dfdx_`margin') \ e(se_`margin')
			}
			local dummy `e(dummy)'
		}
		else if "`e(cmd)'"=="truncreg" {
			capture confirm matrix e(dfdx)
			if _rc==0 {
				tempname V se
				mat `V' = e(V_dfdx)
				forv k= 1/`=rowsof(`V')' {
					mat `se' = nullmat(`se') , sqrt(`V'[`k',`k'])
				}
				mat `dfdx' = e(dfdx) \ `se'
			}
		}
		capture confirm matrix `dfdx'
		if _rc==0 {
			local rnames: rownames `B'
			if `"`meqs'"'!="" local reqs: roweq `B', q
			local i 1
			foreach row of loc rnames {
				if `"`meqs'"'!="" {
					local eq: word `i' of `reqs'
				}
				local col = colnumb(`dfdx',"`row'")
				if `col'>=. | !`:list eq in meqs' {
					mat `B'[`i',`j'*2-1] = .z
					mat `B'[`i',`j'*2] = .z
				}
				else {
					mat `B'[`i',`j'*2-1] =`dfdx'[1,`col']
					mat `B'[`i',`j'*2] = (`dfdx'[2,`col'])^2
					if "`:word `col' of `dummy''"=="1" mat `D'[`i',1] = 1
				}
				local ++i
			}
		}
end

program GetValues
	args v j B _v eqmatch
	capture confirm matrix e(`v')
	if !_rc {
		tempname V
		matrix `V' = e(`v')
		local eqs: coleq `V', q
		local eqs: list clean eqs
		local eqs: list uniq eqs
		if "`eqs'"=="_" {
			local useeqs 0
		}
		else {
			local useeqs 1
			local eqs: roweq `_v', q
			foreach eqm of local eqmatch {
				local eq1: word 1 of `eqm'
				local eq2: word 2 of `eqm'
				local eqs: subinstr local eqs `"`eq2'"' `"`eq1'"', word all
			}
		}
		local vars: rownames `_v'
		local i 0
		foreach var of local vars {
			local ++i
			if `useeqs' local eq: word `i' of `eqs'
			else local eq "_"
			local c=colnumb(`V',`"`eq':`var'"')
			if `c'<. & `B'[`i',`j'*2-1]<.z {
				mat `_v'[`i',`j'] = `V'[1,`c']
			}
		}
	}
end

program GetStats
	args j mestats eqmatch stats
	local eqs: coleq e(b), q
	local eqs: list clean eqs
	local eqs: list uniq eqs
	foreach eqm of local eqmatch {
		local eq1: word 1 of `eqm'
		local eq2: word 2 of `eqm'
		local eqs: subinstr local eqs `"`eq1'"' `"`eq2'"', word
	}
	local e 0
	foreach eq of local eqs {
		local ++e
		foreach stat of local stats {
			local r = rownumb(`mestats',`"`eq':`stat'"')
			if `r'==. continue, break
			if e(cmd)=="mvreg" {
				if "`stat'"=="p" local value: word `e' of `e(p_F)'
				else local value: word `e' of `e(`stat')'
			}
			else if "`stat'"=="df_m" {
				local value "`e(`stat'`e')'"
			}
			else {
				local value "`e(`stat'_`e')'"
			}
			capture confirm number `value'
			if !_rc mat `mestats'[`r',`j'] = `value'
		}
	}
end

program _estout_b
	args B _v eformlist
	local C = colsof(`_v')
	local R = rowsof(`_v')
	forv c = 1/`C' {
		local eform: word `c' of `eformlist'
		forv r = 1/`R' {
			local b `B'[`r',`c'*2-1]
			if "`eform'"=="1" & `b'<. mat `_v'[`r',`c'] = exp(`b')
			else mat `_v'[`r',`c'] = `b'
		}
	}
end

program _estout_se
	args B _v eformlist
	local C = colsof(`_v')
	local R = rowsof(`_v')
	forv c = 1/`C' {
		local eform: word `c' of `eformlist'
		forv r = 1/`R' {
			local b `B'[`r',`c'*2-1]
			local var `B'[`r',`c'*2]
			if `b'>0 & `var'==0 mat `_v'[`r',`c'] = .
			else if `var'>=. mat `_v'[`r',`c'] = `var'
			else if "`eform'"=="1" mat `_v'[`r',`c'] = sqrt(`var') * exp(`b')
			else mat `_v'[`r',`c'] = sqrt(`var')
		}
	}
end

program _estout_t
	args B _v abs
	local C = colsof(`_v')
	local R = rowsof(`_v')
	forv c = 1/`C' {
		forv r = 1/`R' {
			local b `B'[`r',`c'*2-1]
			local var `B'[`r',`c'*2]
			if `var'==0 mat `_v'[`r',`c'] = .
			else if `var'>=. mat `_v'[`r',`c'] = `var'
			else mat `_v'[`r',`c'] = `abs'(`b'/sqrt(`var'))
		}
	}
end

program _estout_p
	args B _v df_r
	local C = colsof(`_v')
	local R = rowsof(`_v')
	forv c = 1/`C' {
		forv r = 1/`R' {
			local b `B'[`r',`c'*2-1]
			local var `B'[`r',`c'*2]
			if `var'==0 mat `_v'[`r',`c'] = .
			else if `var'>=. mat `_v'[`r',`c'] = `var'
			else if `df_r'[1,`c']<. ///
			 mat `_v'[`r',`c'] = ttail(`df_r'[1,`c'],abs(`b'/sqrt(`var'))) * 2
			else mat `_v'[`r',`c'] = (1-norm(abs(`b'/sqrt(`var')))) * 2
		}
	}
end

program _estout_ci
	args sign B _v df_r level eformlist
	local C = colsof(`_v')
	local R = rowsof(`_v')
	forv c = 1/`C' {
		local eform: word `c' of `eformlist'
		forv r = 1/`R' {
			local b `B'[`r',`c'*2-1]
			local var `B'[`r',`c'*2]
			if `var'==0 mat `_v'[`r',`c'] = .
			else if `var'>=. mat `_v'[`r',`c'] = `var'
			else if `df_r'[1,`c']<. mat `_v'[`r',`c'] = `b' `sign' ///
			 invttail(`df_r'[1,`c'],(100-`level')/200) * sqrt(`var')
			else mat `_v'[`r',`c'] = `b' `sign' ///
			 invnorm(1-(100-`level')/200) * sqrt(`var')
			if "`eform'"=="1" & `_v'[`r',`c']<. mat `_v'[`r',`c'] = exp(`_v'[`r',`c'])
		}
	}
end

program _estout_ci_l
	_estout_ci - `0'
end

program _estout_ci_u
	_estout_ci + `0'
end

program NumberMlabels
	args M mlabels
	forv m = 1/`M' {
		local num "(`m')"
		local lab: word `m' of `macval(mlabels)'
		if `"`macval(lab)'"'!="" {
			local lab `"`num' `macval(lab)'"'
		}
		else local lab `num'
		local labels `"`macval(labels)'`"`macval(lab)'"' "'
	}
	c_local mlabels `"`macval(labels)'"'
end

program ModelEqCheck
	args B eq m
	tempname Bsub
	mat `Bsub' = `B'["`eq':",`m'*2-1]
	local R = rowsof(`Bsub')
	local value 0
	forv r = 1/`R' {
		if `Bsub'[`r',1]<. {
			local value 1
			continue, break
		}
	}
	c_local value `value'
end

program Add2Vblock
	args block col
	foreach v of local col {
		gettoken row block: block
		local row "`row' `v'"
		local row: list retok row
		local vblock `"`vblock'"`row'" "'
	}
	c_local vblock `"`vblock'"'
end

program VblockCheck
	args block
	foreach row of local block {
		local norow 1
		foreach v of local row {
			if index("`v'",".")!=1 {
				local norow 0
				continue, break
			}
		}
		if !`norow' {
			local vblock `"`vblock'"`row'" "'
		}
	}
	local vblock: list retok vblock
	c_local vblock `"`vblock'"'
end

program InsertAtVariables
	args value span spanonly M title discrete starlegend
	if `spanonly' local atvars span
	else local atvars span M title discrete starlegend
	foreach atvar of local atvars {
		local value: subinstr local value "@`atvar'" `"`macval(`atvar')'"', all
	}
	c_local value `"`macval(value)'"'
end

program Abbrev
	args width value abbrev
	if "`abbrev'"!="" {
		if `width'>32 {
			local value = substr(`"`macval(value)'"',1,`width')
		}
		else if `width'>0 {
			local value = abbrev(`"`macval(value)'"',`width')
		}
	}
	c_local value `"`macval(value)'"'
end

program MgroupsPattern
	args mrow pattern
	local i 0
	local m0 0
	local j 0
	foreach m of local mrow {
		if `m'!=`m0' {
			local p: word `++i' of `pattern'
			if `i'==1 local p 1
			if "`p'"=="1" local j = `j' + 1
		}
	local newpattern `newpattern' `j'
	local m0 `m'
	}
	c_local mgroupspattern `newpattern'
end

program WriteCaption
	args file delimiter stardetach row rowtwo labels starsrow span  ///
	 abbrev colwidth colfmt delwidth starwidth repeat prefix suffix
	local c 0
	local nspan 0
	local c0 2
	local spanwidth -`delwidth'
	local spanfmt
	foreach r of local row {
		local rtwo: word `=`c'+1' of `rowtwo'
		if `"`span'"'=="" {
			capture local lastrtwo: word `c' of `rowtwo'
			capture local lastr: word `c' of `row'
			if _rc | "`r'"!="`lastr'" | "`rtwo'"!="`lastrtwo'" | `"`rowtwo'"'=="" {
				local value: word `r' of `macval(labels)'
				local value `"`macval(prefix)'`macval(value)'`macval(suffix)'"'
				InsertAtVariables `"`macval(value)'"' "1" 1
				Abbrev `colwidth' `"`macval(value)'"' "`abbrev'"
			}
			else local value
			file write `file' `macval(delimiter)' `colfmt' (`"`macval(value)'"')
			if `:word `++c' of `starsrow''==1 {
				file write `file' `macval(stardetach)' _skip(`starwidth')
			}
		}
		else {
			local ++nspan
			local spanwidth=`spanwidth'+`colwidth'+`delwidth'
			if `:word `++c' of `starsrow''==1 {
				local spanwidth = `spanwidth' + `starwidth'
				if `"`macval(stardetach)'"'!="" {
					local ++nspan
					local spanwidth = `spanwidth' + `delwidth'
				}
			}
			local nextrtwo: word `=`c'+1' of `rowtwo'
			local nextr: word `=`c'+1' of `row'
			if "`r'"!="`nextr'" | "`rtwo'"!="`nextrtwo'" | `"`rowtwo'"'=="" {
				local value: word `r' of `macval(labels)'
				local value `"`macval(prefix)'`macval(value)'`macval(suffix)'"'
				InsertAtVariables `"`macval(value)'"' "`nspan'" 1
				Abbrev `spanwidth' `"`macval(value)'"' "`abbrev'"
				if `spanwidth'>0 local spanfmt "%-`spanwidth's"
				file write `file' `macval(delimiter)' `spanfmt' (`"`macval(value)'"')
				InsertAtVariables `"`macval(repeat)'"' "`c0'-`=`c0'+`nspan'-1'" 1
				local repeatlist `"`macval(repeatlist)'`macval(value)'"'
				local c0 = `c0' + `nspan'
				local nspan 0
				local spanwidth -`delwidth'
			}
		}
	}
	c_local value `"`macval(repeatlist)'"'
end

program WriteEqrow
	args file delimiter stardetach value row span vwidth fmt_v ///
	 abbrev mwidth fmt_m delwidth starwidth prefix suffix
	local nspan 1
	local spanwidth `vwidth'
	local spanfmt
	local value `"`macval(prefix)'`macval(value)'`macval(suffix)'"'
	if `"`span'"'=="" {
		InsertAtVariables `"`macval(value)'"' "1" 1
		Abbrev `vwidth' `"`macval(value)'"' "`abbrev'"
		file write `file' `fmt_v' (`"`macval(value)'"')
		foreach r of local row {
			file write `file' `macval(delimiter)' `fmt_m' ("")
			if `r'==1 {
				file write `file' `macval(stardetach)' _skip(`starwidth')
			}
		}
	}
	else {
		foreach r of local row {
			local ++nspan
			local spanwidth = `spanwidth' + `mwidth' + `delwidth'
			if `r'==1 {
				local spanwidth = `spanwidth' + `starwidth'
				if `"`macval(stardetach)'"'!="" {
					local ++nspan
					local spanwidth = `spanwidth' + `delwidth'
				}
			}
		}
		InsertAtVariables `"`macval(value)'"' "`nspan'" 1
		Abbrev `spanwidth' `"`macval(value)'"' "`abbrev'"
		if `spanwidth'>0 local spanfmt "%-`spanwidth's"
		file write `file' `spanfmt' (`"`macval(value)'"')
	}
end

program VarInList
	args var unstack eqvar eq list
	local value
	local L: word count `macval(list)'
	forv l = 1(2)`L' {
		local lvar: word `l' of `macval(list)'
		local lab: word `=`l'+1' of `macval(list)'
		if "`unstack'"!="" {
			if "`var'"==`"`lvar'"' {
				local value `"`macval(lab)'"'
				continue, break
			}
		}
		else {
			if inlist(`"`lvar'"',"`var'",`"`eqvar'"',`"`eq':"') {
				local value `"`macval(lab)'"'
				continue, break
			}
		}
	}
	c_local value `"`macval(value)'"'
end

program vFormat
	args value fmt lz dmarker msign par
	local value: di `fmt' `value'
	local value: list retok value
	if "`lz'"=="" {
		if index("`value'","0.")==1 | index("`value'","-0.") {
			local value: subinstr local value "0." "."
		}
	}
	if `"`macval(dmarker)'"'!="" {
		if "`: set dp'"=="comma" local dp ,
		else local dp .
		local val: subinstr local value "`dp'" `"`macval(dmarker)'"'
	}
	else local val `"`value'"'
	if `"`msign'"'!="" {
		if index("`value'","-")==1 {
			local val: subinstr local val "-" `"`macval(msign)'"'
		}
	}
	if `"`par'"'!="" {
		tokenize `"`macval(par)'"'
		local val `"`macval(1)'`macval(val)'`macval(2)'"'
	}
	c_local value `"`macval(val)'"'
end

program Stars
	args starlevels P
	if `P'<. {
		local nstar: word count `macval(starlevels)'
		local i `nstar'
		while `i'>1 {
			local istarsym: word `=`i'-1' of `macval(starlevels)'
			local istar: word `i' of `macval(starlevels)'
			if `P'<`istar' {
				local value "`macval(istarsym)'"
				continue, break
			}
			local i = `i' - 2
		}
	}
	c_local value `macval(value)'
end

program Keep   //adapted from est_table.ado
	args b spec
	tempname bt b2
	mat `b2' = `b'
	foreach sp of local spec {
		local isp = rownumb(`b2', "`sp'")
		if `isp'==. {
			dis as err "coefficient `sp' does not occur in any of the models"
			exit 198
		}
		if index("`sp'",":")>0 {
			mat `bt' = nullmat(`bt') \ `b2'["`sp'",1...]
		}
		else {
			while `isp'!=. {
				local nb = rowsof(`b2')
				mat `bt' = nullmat(`bt') \ `b2'[`isp',1...]
				if `isp'==1 {
					mat `b2' = `b2'[2...,1...]
				}
				else if `isp'==`nb' {
					mat `b2' = `b2'[1..`=`nb'-1',1...]
				}
				else {
					local im1 = `isp'-1
					local ip1 = `isp'+1
					mat `b2' = `b2'[1..`im1',1...] \ `b2'[`ip1'...,1...]
				}
				local isp = rownumb(`b2', "`sp'")
			}
		}
	}
	c_local temp `"`: rowfullnames `bt''"'
	capture mat drop `bt' `b2'
end

program Drop   //adapted from est_table.ado
	args b spec
	tempname bt
	mat `bt' = `b'
	foreach sp of local spec {
		local isp = rownumb(`bt', "`sp'")
		if `isp'==. {
			dis as err "coefficient `sp' does not occur in any of the models"
			exit 198
		}
		while `isp'!=. {
			local nb = rowsof(`bt')
			if `isp'==1 {
				mat `bt' = `bt'[2...,1...]
			}
			else if `isp'==`nb' {
				mat `bt' = `bt'[1..`=`nb'-1',1...]
			}
			else {
				local im1 = `isp'-1
				local ip1 = `isp'+1
				mat `bt' = `bt'[1..`im1',1...] \ `bt'[`ip1'...,1...]
			}
			local isp = rownumb(`bt', "`sp'")
		}
	}
	c_local temp `"`: rowfullnames `bt''"'
	capture mat drop `bt'
end
