*! full information maximum liklihood  30Jan04  Chris Metli

program define fiml, sortpreserve eclass

	version 8.0

	syntax varlist(min=2) [if] [in], fixedeffect(varname) [EPsilon(real 10e-6) /*
		*/ ALpha(name) FITted(name) RESidual(name)]

	* set up tempnames, scalars, locals, tempvars
	tempname teststat ll_r beta betaold XpX Xpy
	scalar `teststat' = 1
	local yvar : word 1 of `varlist'
	local indvarlist : subinstr local varlist "`yvar'" ""
	tempvar `yvar'bar `yvar'diff sigma2 tvar lnlh
	if "`alpha'"=="" tempvar alpha
	else confirm new variable `alpha'
	if "`fitted'"=="" tempvar fitted
	else confirm new variable `fitted'
	if "`residual'"=="" tempvar residual
	else confirm new variable `residual'
	
	* mark sample and preserve
	marksample touse
	preserve
	
	display _newline(0)
	quietly {
	
		* calculate differenced variables
		sort `touse' `fixedeffect'
		foreach var of local varlist {
			tempvar `var'bar `var'diff
			by `touse' `fixedeffect': egen ``var'bar' = mean(`var') if `touse'
			generate ``var'diff' = `var' - ``var'bar' if `touse'
		}
		
		* geneate blank variables and initialize beta matrix by running areg
		generate `sigma2' = .
		generate `tvar' = .
		generate `lnlh' = .
		count if `touse'
		local T = r(N)
		regress `varlist' if `touse'
		matrix `beta' = e(b)'
		scalar `ll_r' = e(ll)
		ereturn clear
		local num = rowsof(`beta') - 1
		matrix `beta' = `beta'[1..`num',1]
		local toploop = rowsof(`beta')
		local itcount = 0
		
		
		* loop until sum of difference of betas across runs is less than epsilon
		while `teststat' > `epsilon' {
		
			* calculate sigma squared
			replace `tvar' = ``yvar'diff' if `touse'
			forvalues i = 1/`toploop' {
				local var : word `i' of `indvarlist'
				replace `tvar' = `tvar' - `beta'[`i', 1] * ``var'diff' if `touse'
			}
			by `touse' `fixedeffect': replace `sigma2' = sum(`tvar' * `tvar') if `touse'
			by `touse' `fixedeffect': replace `sigma2' = `sigma2'[_N]/_N if `touse'
			
			* accumulate X'X and X'y
			local v1count = 1
			matrix accum `XpX' = `varlist' if `touse', noconstant
			matrix `Xpy' = `XpX'[2...,1]
			matrix `XpX' = `XpX'[2...,2...]
			foreach var1 of local indvarlist {
				local v2count = 1
				foreach var2 of local indvarlist {
					replace `tvar' = sum(``var1'diff' * ``var2'diff' / `sigma2') if `touse'
					matrix `XpX'[`v1count', `v2count++'] = `tvar'[_N]
				}
				replace `tvar' = sum(``var1'diff' * ``yvar'diff' / `sigma2') if `touse'
				matrix `Xpy'[`v1count++', 1] = `tvar'[_N]
			}
			
			* calculate beta vector
			matrix `betaold' = `beta'
			matrix `beta' = syminv(`XpX')*`Xpy'
			
			* calculate log likelihood, and test statistic (sum of new - old betas)
			scalar `teststat' = 0
			forvalues i = 1/`toploop' {
				local var : word `i' of `indvarlist'
				scalar `teststat' = `teststat' + `beta'[`i', 1] - `betaold'[`i', 1]
			}
			by `touse' `fixedeffect': replace `lnlh' = sum(-_N/2 * ln(2*`c(pi)'*`sigma2') * (_n==_N)) if `touse'
			replace `lnlh' = sum(`lnlh') - .5*`T' if `touse'
			
			* display log likelihood
			noisily display as text "Iteration `itcount++':   log likelihood =", as result `lnlh'[_N]
			
		* ends calculation loop
		}
		
		* generate alphas, fitted, & residuals
		generate `alpha' = ``yvar'bar' if `touse'
		forvalues i = 1/`toploop' {
			local var : word `i' of `indvarlist'
			replace `alpha' = `alpha' - `beta'[`i', 1] * ``var'bar' if `touse'
		}
		generate `fitted' = `alpha' if `touse'
		forvalues i = 1/`toploop' {
			local var : word `i' of `indvarlist'
			replace `fitted' = `fitted' + `beta'[`i', 1] * `var' if `touse'
		}
		generate `residual' = `yvar' - `fitted' if `touse'
		
		* calculate results
		matrix `beta' = `beta''
		matrix `XpX' = syminv(`XpX')
		by `touse' `fixedeffect': replace `tvar' = (_n==1) if `touse'
		count if `tvar'
		local dof = `T' - `toploop' - r(N)
		matrix accum `Xpy' = `yvar' if `touse', noconstant
		replace `tvar' = sum(`residual' * `residual') if `touse'
		
	* ends quietly block
	}
	
	* post results
	ereturn post `beta' `XpX', depname("`yvar'") obs(`T') dof(`dof')
	ereturn scalar rss = `tvar'[_N]
	ereturn scalar mss = `Xpy'[1,1] - `tvar'[_N]
	ereturn scalar rmse = sqrt(e(rss)/`dof')
 	ereturn scalar r2 = 1 - e(rss)/`Xpy'[1,1]
	ereturn scalar df_m = e(N) - e(df_r)
	ereturn scalar ll = `lnlh'[_N]
	ereturn scalar ll_r = `ll_r'
	ereturn local depvar "`yvar'"
	ereturn local cmd "fiml"
	display _newline(0)
	ereturn display
	
	restore, not

end
