*! version 1.1   21 oct 2003   author: Chris Metli

program define groupcmd, sortpreserve

	version 8.0

	syntax varlist(min=2 max=2) [if] [in], Generate(name) [Missing]
	
	local var1 : word 1 of `varlist'
	local var2 : word 2 of `varlist'
	local newvar "`generate'"

	* error checking
	if "`var1'"=="`var2'" {
		disp as error "two distinct variables must be specified"
		exit
	}
	confirm new variable `newvar'

	* declare temporary files and variables
	tempfile tlink
	tempvar linkvar obs dummy merge

	* mark the sample
	tempvar touse
	mark `touse' `if' `in'
	if "`missing'"=="" markout `touse' `varlist', strok

	* preserve dataset
	preserve

	* start quietly block
	quietly {

		* keep only necessary variables and observations
		keep if `touse'
		keep `var1' `var2'
		duplicates drop

		* save basefile with grouping variables
		save "`tlink'"

		* replicate first variable to start loop
		drop `var2'
		generate `linkvar' = `var1'

		* intialize loop test and counter
		local loop = 0
		local test = -1

		* execute double joinby loop until don't get anymore expansion
		while r(N)!=`test' {

			* first part of loop test
			count
			if r(N)==0 {
				noisily display as error "no observations in if/in criteria"
				exit
			}
			local test = r(N)

			* joinby on itself using 1st variable and keep unique
			joinby `var1' using "`tlink'"
			drop `var1'
			duplicates drop

			* joinby on itself using 2nd variable and keep unique
			joinby `var2' using "`tlink'"
			drop `var2'
			duplicates drop

			* loop counter and second part of loop test
			local loop = `loop' + 1
			count

		}

		* reshape out wide all of the linked variables to the var1
		* (really just a bunch of var1s associated with each other)
		sort `var1' `linkvar'
		by `var1': generate `obs' = _n
		reshape wide `linkvar', i(`var1') j(`obs')

		* drop var1 (will be included in one of the wide linkvars) and 
		* keep unique relationships between linkvars
		drop `var1'
		bysort `linkvar'*: keep if _n==1
		
		* generate grouping by the unique relationships and reshape to associate the
		* grouping variable with all of the linkvars (all associated var1s)
		generate `newvar' = _n
		reshape long `linkvar', i(`newvar') j(`dummy')
		
		* drop empties (due to variable number of var1s in a grouping)
		capture drop if `linkvar'==.
		capture drop if `linkvar'==""
		
		* rename linkvar, order, sort, and save
		rename `linkvar' `var1'
		order `var1' `newvar'
		sort `var1'
		save "`tlink'", replace

		* open original dataset and merge on grouping information
		restore, preserve
		sort `var1'
		merge `var1' using "`tlink'", uniqusing _merge(`merge')

	}

	* check to make sure merge went correctly
	quietly count if (`merge'!=1 & `newvar'==.) | (`merge'==1 & `newvar'!=.)
	if r(N)>0 {
		display as error "program merge error"
		exit
	}
	quietly count if `newvar'==. & `touse'
	if r(N)>0 {
		display as error "program merge error"
		exit
	}

	* replace groupings for observations didn't want grouped
	quietly replace `newvar' = . if !(`touse')

	* display number of loops needed
	display as result "number of links deep: `loop'"

	* clear restore
	restore, not

end


