*! version 7.0   23oct02

capture program drop namemerge
program define namemerge

	version 7.0
	syntax anything(name=vars id="varlist") using/ [, UPDATE REPLACE /*
		*/ REFORmat ANYorder THorough numvars(string) cutoff(numlist max=1) /*
		*/ stop(integer 1) _loop(string) _merge(string) ]

	local varcount : word count `vars'
	if `varcount' > 2 {error 103}
	else if `varcount' < 2 {error 102}

	local var1 : word 1 of `vars'
	local var2 : word 2 of `vars'

	if "`var1'"=="`var2'" {
		display as error "variable names must be different"
		exit 198
	}

	* check for loaded data
	quietly count
	if r(N)==0 {
		display as error "no observations in master dataset"
		exit 3
	}

	if "`_merge'"=="" {local _merge "_merge"}
	if "`_loop'"=="" {local _loop "_loop"}

	* parse numvars comparison variables
	if "`numvars'"~="" {

		local varcount : word count `numvars'
		if `varcount' > 2 {
			disp as error "too many variables specified for numvars option"
			exit 103
		}
		else if `varcount' < 2 {
			disp as error "too few variables specified for numvars option"
			exit 102
		}

		local nvar1 : word 1 of `numvars'
		local nvar2 : word 2 of `numvars'

		if "`nvar1'"=="`nvar2'" {
			display as error "numvars option variable names must be different"
			exit 198
		}

		* make sure there is a cutoff value
		if "`cutoff'"=="" {
			display as error "must specify a percentage cutoff value (ie. 30% = 30)"
			exit 198
		}
		if `cutoff'<0 {
			display as error "percentage cutoff value must be nonnegative (ie. 30% = 30)"
			exit 198
		}
	}

	* cannot have cutoff value without numvars variables specified
	if "`cutoff'"~="" & "`numvars'"=="" {
		display as error "cannot specify a cutoff value without numerical variables to compare"
		exit 198
	}

	* anyorder implies reformat (although reformat can exist without anyorder)
	if "`anyorder'"=="anyorder" {local reformat "reformat"}

	* set up temporary objects
	tempfile file1 file2 sfile1 sfile2 fullmatches listforchecking anytemp anytemp2
	tempvar mergevar length oldvar index
	tempname l1 l2 count1 count2

	* confirm using file and add .dta if necessary
	capture confirm file "`using'"
	if _rc ~= 0 {
		if index("`using'",".")==0 {
			local using "`using'.dta"
		}
	}
	confirm file "`using'"

	preserve

	* format the datasets for merging
	forvalues j = 1/2 {

		if `j'==2 {
			use "`using'"
			* check for loaded data
			quietly count
			if r(N)==0 {
				display as error "no observations in using dataset"
				exit 3
			}
		}

		confirm new var `_merge'
		confirm new var `_loop'

		* record variable order
		unab varlist`j' : *

		* take out merge variables from variable order lists
		tokenize `varlist`j''
		local varlist`j' ""
		while "`1'"~="" {
			if "`1'"~="`var`j''" {local varlist`j' "`varlist`j'' `1'"}
			macro shift
		}

		* confirm required merge variables
		capture confirm variable `var`j''
		if _rc ~= 0 {
			if `j'==1 {display as error "variable `var`j'' not found in master data"}
			if `j'==2 {display as error "variable `var`j'' not found in using data"}
			exit = 111
		}
		confirm string variable `var`j''

		* initialize variable lists to use with the string merge variable
		local vlist`j' "`var`j''"

		* confirm numvars option variables
		if "`numvars'"~="" {
			capture confirm variable `nvar`j''
			if _rc ~= 0 {
				if `j'==1 {display as error "variable `nvar`j'' not found in master data"}
				if `j'==2 {display as error "variable `nvar`j'' not found in using data"}
				exit 111
			}
			confirm numeric variable `nvar`j''

			* add numvars variables to variable lists to use
			local vlist`j' "`vlist`j'' `nvar`j''"
		}

		* create original dataset with group number, and smaller dataset with relevant vars
		quietly {
			tempvar group`j'
			egen `group`j'' = group(`vlist`j''), missing
			sort `group`j''
			save `sfile`j''

			keep `vlist`j'' `group`j''
			bysort `group`j'': keep if _n==1

			save `file`j'', replace

			* record longest string variable
			generate `length' = length(`var`j'')
			summarize `length', meanonly
			drop `length'
			local tl = r(max)
			scalar `l`j'' = `tl'
			save `file`j'', replace

			* generate new variables with removed formating if reformat option selected
			if "`reformat'"=="reformat" {

				generate str`tl' `oldvar' = `var`j''
				replace `oldvar' = trim(subinstr(`oldvar',"`"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"~"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"!"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"@"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"#"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"$"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"%"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"^"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"&"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"*"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"("," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',")"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"-"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"_"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"+"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"="," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"["," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"]"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"{"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"}"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"|"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"/"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"\"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',";"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',":"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"'"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"<"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',">"," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"."," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',","," ",.))
				replace `oldvar' = trim(subinstr(`oldvar',"?"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"AND"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"THE"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"OF"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"INC"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"CORP"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"CO"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"FSB"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"NA"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"NB"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"GROUP"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"GRP"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"HOLDINGS"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"HLDGS"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"LTD"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"LLC"," ",.))
				replace `oldvar' = trim(subinword(upper(`oldvar'),"ADR"," ",.))

				* cut down extra spaces in the middle of words
				local tl2 = `tl'/2
				forvalues n = 1/`tl2' {
					replace `oldvar' = subinstr(`oldvar',"  "," ",.)
				}

				keep `vlist`j'' `group`j'' `oldvar'
				drop `var`j''
				save `anytemp', replace
				rename `oldvar' `var`j''
				append using `file`j''
				save `file`j'', replace

			} /* ends reformat option check */

			* break out each string variable into parts if anyorder option selected
			if "`anyorder'"=="anyorder" {

				while 1 {
					use `anytemp'

					* drop if length is 2 or less - too small to care
					* (will still have original if original is 2 or less though)
					drop if length(`oldvar') <= 2

					capture drop `var`j''
					generate `index' = index(`oldvar'," ")
					save `anytemp2', replace

					* append on the observations that are only 1 word long
					keep if `index' == 0
					rename `oldvar' `var`j''
					drop `index'
					append using `file`j''
					save `file`j'', replace

					use `anytemp2'
					drop if `index' == 0

					* check for end of process - everything down to 1 word
					count
					if r(N)==0 {
						use `file`j'', clear
						drop `oldvar'
						save `file`j'', replace
						continue, break
					}

					* pull out first word
					generate str`tl' `var`j'' = trim(substr(`oldvar',1,`index'-1))
					replace `oldvar' = trim(substr(`oldvar',`index',.))

					* drop if length is 2 or less - too small to care
					* (will still have original if original is 2 or less though)
					drop if length(`var`j'') <= 2

					drop `index'
					save `anytemp', replace

					append using `file`j''
					save `file`j'', replace

				} /* ends while loop */
			} /* ends anyorder option check */

		} /* ends quietly block */
	} /* ends formatting j loop */

	* record longest length - start from there
	if `l1' > `l2' {local length = `l1'}
	else {local length = `l2'}

	* loop over string lengths from longest length till stop value (or until completion)
	quietly {
	forvalues i = `length'(-1)`stop' {

		/* formats each list to merge */
		forvalues k = 1/2 {
			use "`file`k''", clear
			bysort `vlist`k'' `group`k'': drop if _n~=1
			capture drop `mergevar'
			generate str`i' `mergevar'=substr(upper(trim(`var`k'')), 1, `i')
			sort `mergevar'
			save "`file`k''", replace
			count
			scalar `count`k'' = r(N)
		}

		* exit if either variable merged completely
		if `count1'==0 | `count2'==0 {continue, break}

		* merges the matches of length i into fullMatches, appends to ListForChecking
		joinby `mergevar' using "`file1'", unmatched(both) _merge(`_merge')
		keep if `_merge' == 3

		* test numvars variables % difference, if applicable
		if "`numvars'"~="" {
			tempvar p_diff
			if `nvar1'<`nvar2' {generate `p_diff' = abs((`nvar1' - `nvar2')/ /*
				*/ abs(`nvar1')) * 100}
			else {generate `p_diff' = abs((`nvar1' - `nvar2')/ /*
				*/ abs(`nvar2')) * 100}
			drop if `p_diff' > `cutoff' & `p_diff'~=.
			drop `p_diff'
		}

		keep `group1' `group2'
		bysort `group1' `group2': drop if _n~=1
		sort `group1'
		save "`fullmatches'", replace

		generate byte `_loop' = `i'

		if `i' == `length' {
			save "`listforchecking'", replace
		}
		else {
			append using "`listforchecking'"
			sort `group1' `group2' `_loop'
			by `group1' `group2': drop if _n~=_N
			save "`listforchecking'", replace
		}

		* takes out matches from both datasets, unless using thorough option
		if "`thorough'"~="thorough" {

			use "`file1'"
			sort `group1'
			merge `group1' using "`fullmatches'", _merge(`_merge')
			drop if `_merge' == 3
			drop `_merge' `group2'
			save "`file1'", replace

			use "`fullmatches'"
			sort `group2'
			save "`fullmatches'", replace

			use "`file2'"
			sort `group2'
			merge `group2' using "`fullmatches'", _merge(`_merge')
			drop if `_merge' == 3
			drop `_merge' `group1'
			save "`file2'", replace
		}

	} /* ends i length loop */
	} /* ends quietly block */

	use "`listforchecking'"

	sort `group1'
	tempvar merge1
	joinby `group1' using `sfile1', unmatched(both) _merge(`merge1')
	sort `group2'
	joinby `group2' using `sfile2', unmatched(both) `update' `replace' _merge(`_merge')
	quietly replace `_merge' = 1 if `merge1' == 2
	drop `merge1' `group1' `group2'

	label value `_merge'
	quietly compress `var1' `var2' `_loop' `_merge'
	format %4.0g `_loop' `_merge'

	order `var1' `var2' `varlist1' `varlist2' `_loop' `_merge'
	gsort `var1' -`_loop' `var2'
	summarize `_loop', meanonly

	display as text "(string length looped from `length' to `r(min)')"

	restore, not

end /* end program namemerge */
