*! risumm.ado
*! Jeff Arnold
*! 30Jan2007
* Differences in this version
* Implicate variable must be specified
* Fixed bugs in number of obs and weighted summ.

program define risumm, rclass byable(recall) 
version 8.2
set more off

syntax [varlist] [aweight fweight iweight pweight] [if] [in], imp(string) [Detail]
preserve
marksample touse


if "`detail'" == "" {
    /* This creates a table similar to a normal summarize statement */
    display
    display in text "    Variable {c |}" _col(20)"Obs" _s(6)"Weight" _s(8)"Mean" ///
	 _s(3)"Std. Dev." _s(7)"Min" _s(8)"Max"
    display in text "{hline 13}{c +}{hline 65}"
}

* need to figure out some way to loop through implicates;
tempvar I
egen `I' = group(`imp') if `imp' ~= .
summ `I', meanonly
local m = r(max)

foreach var of local varlist {

	quietly summ `var' [`weight' `exp'] if `touse', meanonly

        // observation number fixed from last version
        // previously incorectly calculated values from last variable specified
	local n = r(N)
	local sum_w = r(sum_w)

        * summing summary statstics from each implicate
	forvalues i == 1/`m' {
           if (`i' == 1) local plus ""
           else local plus "+"
           quietly summarize `var' [`weight' `exp'] if `I'==`i', detail
	   local Q`i' = r(mean)
	   local Qbar = `Qbar' `plus' `Q`i''
	   local Ubar = `Ubar' `plus' r(Var) 
	   local p1 = `p1' `plus'  r(p1)
	   local p5 = `p5' `plus' r(p5)
	   local p10 = `p10' `plus' r(p10)
	   local p25 = `p25' `plus' r(p25)
	   local p50 = `p50' `plus' r(p50)
	   local p75 = `p75' `plus' r(p75)
	   local p90 = `p90' `plus' r(p90)
	   local p95 = `p95' `plus' r(p95)
	   local p99 = `p99' `plus' r(p99)
	   local max = `max' `plus' r(max)
	   local min = `min' `plus' r(min)
	}

	* divide by number of implicates to get mean of each statistic
	foreach j in Qbar Ubar p1 p5 p10 p25 p50 p75 p90 p95 p99 min max {
	    local `j' = (``j'')/`m'
        }

	* Calculate Between imputation variation 
        local B = 0
	forvalues i = 1/`m' {
	    local B = `B' + (`Q`i'' - `Qbar')^2
	}
	local B = (1/(1-`m'))*`B'
	* Add within and between imputation variances to get total variance
	local T= `Ubar' + (1+(1/`m'))*`B'
	* standard deviation 
	local sd = sqrt(`T')

	if "`detail'" ~= "" { 
	    display
	    di as text %~61s "`var'"
	    di as text "{hline 61}" 
	    di as text _s(6) "Percentiles" 
	    di as text "Min" as result %9.0g _col(8) `min' ///
		as text _col(24)"Obs" as result _col(39) %9.0g `n'  
	    di as text " 1%" as result %9.0g _col(8) `p1' ///
		as text _col(24)"Sum of Wgt." as result _col(39) %9.0g `sum_w'    
	    di as text " 5%" as result %9.0g _col(8) `p5'    
	    di as text "10%" as result %9.0g _col(8) `p10' ///
		as text _col(24)"Mean" as result _col(39) %9.0g `Qbar' 
	    di as text "25%" as result %9.0g _col(8) `p25' ///
		as text _col(24)"Std. Dev." as result _col(39) %9.0g `sd'
	    display
	    di as text "50%" as result %9.0g _col(8) `p50' ///
		as text _col(24) "Total Variance" as result _col(39) %9.0g `T'  
	    di as text  _col(24)"Within Var." as result _col(39) %9.0g `Ubar'
	    di as text "75%" as result %9.0g _col(8) `p75' ///
		as text _col(24) "Between Var." as result _col(39) %9.0g `B'
	    di as text "90%" as result %9.0g _col(8) `p90'  
	    di as text  "95%" as result %9.0g _col(8) `p95'
	    di as text  "99%" as result %9.0g _col(8) `p99'
	    di as text  "Max" as result %9.0g _col(8) `max'
	}
	else {
	    display in text %12s abbrev("`var'",12) " {c |}" as result ///
		    %8.0g `n' _s(3) ///
		    %9.0g `sum_w' _s(3) ///
		    %9.0g `Qbar' _s(2) ///
		    %9.0g `sd' _s(2) ///
		    %9.0g `min' _s(2) ///
		    %9.0g `max'
	}

	return scalar p99 = `p99'
	return scalar p95 = `p95'
	return scalar p90 = `p90'
	return scalar p75 = `p75'
	return scalar p50 = `p50'
	return scalar p25 = `p25'
	return scalar p10 = `p10'
	return scalar p5 = `p5'
	return scalar p1 = `p1'
	return scalar max = `max'    
	return scalar min = `min'   
	return scalar B = `B'
	return scalar U = `Ubar'
	return scalar var = `T'
	return scalar sd = `sd'
	return scalar mean = `Qbar'
	return scalar sum_w = `sum_w' 
	return scalar N = `n'
}

end
