*! seeout
*! version 1.1August 28, 2007 @ 09:24:51*! October 2005
*! by roywada@hotmail.com
*
* (to accompany -outreg2-)
*
* version 1.1 replaced rowmiss option in egen, which is not available in 8.2



program define seeout
version 8.2

syntax [using]

if "`using'"~="" {
	*** clean up file name, attach .txt if no file type is specified
	local rest "`using'"
	* strip off "using"
	gettoken part rest: rest, parse(" ")
	* strip off quotes
	gettoken first second: rest, parse(" ")
	local rest: list clean local(rest)
	
	local rabbit `"""'
	if index(`"`using'"', ".")==0 {
		local file = `"`rabbit'`first'.txt`rabbit'"'
		local using = `"using `file'"'
	}
	else {
		local file = `"`rabbit'`first'`rabbit'"'
		local using = `"using `file'"'
	}
	
	*** seeout the output
	local cl `"{stata `"seeout `pref'"':  seeout `pref'}"'
	di as txt `"`cl'"'
	seeing `using'
}

else {
	*** read the set preference if not out of date

	* NOTE: `0' is written over below
	quietly findfile outreg2_prf.ado
	tempname myfile
	file open `myfile' using `"`r(fn)'"', read text
	file read `myfile' date
	file read `myfile' pref
	file read `myfile' options
	file close `myfile'
	
	if "`date'"== "`c(current_date)'" {
		*** seeout the output
		seeing `pref'
		
		* similar to the other one except the clickable text
		local cl `"{stata `"seeout `pref'"':  seeout `pref'}"'
		di as txt `"`cl'"'
	}
	else {
		di in red "must specify the filename (the last preference has expired)"
		exit 100
	}
}
end



***********************



program define seeing, rclass
version 8.2

quietly{
	* syntax using/[, Clear]
	syntax using

	preserve
	
	insheet `using', nonames clear
	describe, short
	
	* number of columns
	local numcol = r(k)	

	tempvar blanks rowmiss	
	count if v1=="EQUATION"
	if `r(N)'~=0 {
		count if v3=="LABELS"
		if `r(N)'~=0 {
			local num=4
		}
		else {
			local num=3
		}
	}
	else {
		count if v2=="LABELS"
		if `r(N)'~=0 {
			local num=3
		}
		else {
			local num=2
		}
	}

	gen int `blanks' = (trim(v`num')=="")
	forvalues col = `num'/`numcol' {
		replace `blanks' = `blanks' & (trim(v`col')=="")
	}
	
	* in case ctitle is missing: 2 of 2
	* colheadN
	replace `blanks'=0 if v1=="COEFFICIENT" | v1[_n-1]=="COEFFICIENT"
	replace `blanks'=0 if v2=="COEFFICIENT" | v2[_n-1]=="COEFFICIENT"
	
	* title rows
      local titlrow = 0 
      while `blanks'[`titlrow'+1] {
		local titlrow = `titlrow'+1
	}
	
	if `numcol'==2 {
		local colheadN=2+`titlrow' 
	}
	else {
		local colheadN=3+`titlrow'
	}
	
	* statistics rows
	count if `blanks'==0
	local strowN = `r(N)'+`titlrow'
	
	
	* move the notes and titles to the top of a new column
	gen str5 Notes_Titles=""
	format Notes_Titles %-20s 
	count if v1=="EQUATION"
	if `r(N)'==0 {
		* EQUATION column does not exist
		if `titlrow'>0 {
			forval num=1/`titlrow' {
				replace Notes_Titles=v1[`num'] in `num'
				replace v1="" in `num'
			}
		}
		
		local one = 1
		local legend = v1[`strowN'+`one']
		while "`legend'"~="" {
			local place=`strowN'+`one'
			local legend = v1[`place']
			replace Notes_Titles="`legend'" in `=`one'+`titlrow'+1'
			if "`legend'"~="" {
				replace v1="" in `place'
			}
			local one = `one'+1
		}
		* change the string length
		gen str5 temp=""
		replace temp=v1
		drop v1
		ren temp v1
		order v1
		* format
		foreach var of varlist v1 {
			local _format= "`: format `var''"
			local _widths=substr("`_format'",2,length(trim("`_format'"))-2)
			format `var' %-`_widths's
		}
	}
	else {
		* equation column exists
		if `titlrow'>0 {
			forval num=1/`titlrow' {
				replace Notes_Titles=v2[`num'] in `num'
				replace v2="" in `num'
			}
		}
		
		local one = 1
		local legend = v2[`strowN'+`one']
		while "`legend'"~="" {
			local place=`strowN'+`one'
			local legend = v2[`place']
			replace Notes_Titles="`legend'" in `=`one'+`titlrow'+1'
			if "`legend'"~="" {
				replace v2="" in `place'
			}
			local one = `one'+1
		}
		* change the string length
		gen str5 temp=""
		replace temp=v2
		drop v2
		ren temp v2
		order v1 v2
		* format
		foreach var of varlist v1 v2 {
			local _format= "`: format `var''"
			local _widths=substr("`_format'",2,length(trim("`_format'"))-2)
			format `var' %-`_widths's
		}
	}
	
	* clean up
	*egen `rowmiss'=rowmiss(_all)
	* rowmiss option not available in 8.2, do it by hand

	gen `rowmiss'=0
	foreach var of varlist _all {
		if "`var'"~="`rowmiss'" & "`var'"~="`blanks'" {
			replace `rowmiss'=1+`rowmiss' if `var'==""
		}
	}
	
	drop if `rowmiss'==`numcol'+1
	drop `blanks' `rowmiss'
	
	browse
	restore, preserve
}

end  /* end of seeing */











