*! version 1.0January 31, 2008 @ 11:38:02
#delimit ;

program define test2, rclass;
   version 8.0;

   /****************************************************************************
     test2.ad
     Two-sample Wald test.

     This ado-file allows users to run a t-test on H0: estimated beta from most
     recent regression is the same as estimated beta from equation `estimates'.

     Unequal variance methodology was suggested by B. L. Welch (Biometrika v34
     (1/2), 1947) to compare  regression coefficents from the same model
     estimated over two different samples when the underlying populations'
     variances may not be the same.

      - Matt Botsch, 31 January 2008.
   *****************************************************************************/
   syntax varlist(max=1),  /*beta on RH side variable `varlist' to be tested*/
     Estimates(namelist min=1 max=2)
                           /*name of stored estimates*/
     [UNEqual Welch]       /*unequal var Satterthwaite / Welch formual for df */
     [Level(cilevel)]      /*confidence level for reporting */;

   tempname ecurrent df1 u1 sd1 sse1 df2 u2 sd2 sse2;

   gettoken model1 model2 : estimates;
   local model2 "`=ltrim("`model2'")'";
   capture _estimates hold `ecurrent', restore;

   _estimates unhold `model1';

   /*check specification of model1*/
   local y1 = e(depvar);
   local X1 : colnames e(b);
   local test: list varlist in X1;
   if !`test' {;
      di as error "`varlist' not found in `model1'";
      estimates store `model1';
      exit 111;
   };
   /*end check*/

   scalar `df1' = e(df_r);
   scalar `u1' = _b[`varlist'];
   scalar `sd1' = _se[`varlist'];
   scalar `sse1' = e(rss);
   estimates store `model1';

   if "`model2'" != "" {;
      _estimates unhold `model2';

      /*check specification of model2*/
      local y2 = e(depvar);
      local X2 : colnames e(b);
      local test: list varlist in X2;
      if !`test' {;
         di as error "`varlist' not found in `model2'";
         estimates store `model2';
         exit 111;
      };
      /*end check*/

      scalar `df2' = e(df_r);
      scalar `u2' = _b[`varlist'];
      scalar `sd2' = _se[`varlist'];
      scalar `sse2' = e(rss);
      estimates store `model2';
   };

   else {;
      _estimates unhold `ecurrent';

      /*check specification of model2*/
      local y2 = e(depvar);
      local X2 : colnames e(b);
      local test: list varlist in X2;
      if !`test' {;
         di as error "`varlist' not found";
         exit 111;
      };
      /*end check*/

      scalar `df2' = e(df_r);
      scalar `u2' = _b[`varlist'];
      scalar `sd2' = _se[`varlist'];
      scalar `sse2' = e(rss);
   };

   /*check that same model is specified for both samples*/
   local test: list X1 === X2;
   if !`test' {;
      di _n as error "Warning: models use different specifications";
   };
   if "`y1'"!="`y2'" {;
      di _n(`test') as error "Warning: different dependent variables";
   };
   /*end check*/

   TwoTest `df1' `u1' `sd1' `sse1' `df2' `u2' `sd2' `sse2' `level'
     "`unequal'" "`welch'" "`model1'" "`model2'";

   return add;

end;

        
/*modified from ttesti.ado */
program define TwoTest, rclass;
   version 6.0;

   args f1 m1 s1 S2_1 f2 m2 s2 S2_2 level unequal welch mname1 mname2;

   /* Compute statistics. */

   tempname var df t se p pl pr n;

   /* calculate degrees of freedom */
   if "`unequal'"!="" | "`welch'"!="" {;
      local un "un";

      scalar `var' = (`s1')^2 + (`s2')^2;

      if "`welch'"!="" {;
         scalar `df' = -2 + `var'^2
            /( (`s1')^4/(`f1'+2)
             + (`s2')^4/(`f2'+2) );
      };
      else {;
         scalar `df' = `var'^2
            /( (`s1')^4/`f1'
             + (`s2')^4/`f2' );
      };
   };
   else {;
      scalar `df' = `f1' + `f2';
      scalar `var' = (`S2_1' + `S2_2')/(`f1' + `f2')
                     *(`f1'/`S2_1' *`s1'^2 + `f2'/`S2_2' *`s2'^2);
     /* naive approximation: */
     /* scalar `var' = (`f1'*(`s1')^2 + `f2'*(`s2')^2)/(`f1' + `f2');       */
   };

   /*calculate t-stat*/
   scalar `t'  = (`m1'-`m2')/sqrt(`var');
   scalar `se' = sqrt(`var');

   scalar `p' = tprob(`df',abs(`t'));
   if `t' < 0 {;
      scalar `pl' = `p'/2;
      scalar `pr' = 1 - `pl';
   };
   else {;
      scalar `pr' = `p'/2;
      scalar `pl' = 1 - `pr';
   };

  di _n as text "Two-sample t test with `un'equal variances";
  DisplayTable `f1' `m1' `s1' `f2' `m2' `s2' `df' `se' `t' `p'
        `level' "`unequal'" "`welch'" "`mname1'" "`mname2'";

   /* save in r() */
   ret scalar df_2 = `f2';
   ret scalar se_2 = `s2';
   ret scalar b_2  = `m2';
   ret scalar df_1 = `f1';
   ret scalar se_1 = `s1';
   ret scalar b_1  = `m1';
   ret scalar se   = `se';
   ret scalar diff = `m1' - `m2';
   ret scalar p_l  = `pl';
   ret scalar p_u  = `pr';
   ret scalar p    = `p';
   ret scalar df_t = `df';
   ret scalar t    = `t';

end;


program DisplayTable;
   version 6.0;

   args f1 m1 s1 f2 m2 s2 df se t p level unequal welch mname1 mname2;

/*
    5   10   15   20   25   30   35   40   45   50   55   60   65   70   75
++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++|++++
             |       Coef.      Std. Err.        df       [95% Conf. Interval]
-------------+----------------------------------------------------------------
         (b) |   123456789     123456789    1234567      123456789   123456789
*/


/* Display table of coefficient estimates from each sample */

   local cil `=string(`level')';
   local cil `=length("`cil'")';
   local cilabel "Conf. Interval";
   if `cil' == 5 {;
      local cilabel "Conf.Interval";
   };

   di in smcl as text "{hline 13}{c TT}{hline 64}" _n
      _col(14) "{c |}"
      _col(22) "Coef."
      _col(33) "Std. Err."
      _col(50) "df"
      _col(`=61-`cil'+(`cil'==5)') `"[`=strsubdp("`level'")'% `cilabel']"';

   di in smcl as text "{hline 13}{c +}{hline 64}";

   local q = invt(`f1', `level'/100);
   di in smcl as text %12s "(b:`=abbrev("`mname1'",8)')" " {c |}" as result
       _col(18) %9.0g `m1'
       _col(32) %9.0g `s1'
       _col(45) %7.0f `f1'
       _col(58) %9.0g (`m1')-`q'*(`s1')
       _col(70) %9.0g (`m1')+`q'*(`s1') ;

   local q = invt(`f2', `level'/100);
   di in smcl as text %12s "(B:`=abbrev("`mname2'",8)')" " {c |}" as result
       _col(18) %9.0g `m2'
       _col(32) %9.0g `s2'
       _col(45) %7.0f `f2'
       _col(58) %9.0g (`m2')-`q'*(`s2')
       _col(70) %9.0g (`m2')+`q'*(`s2') ;

   di in smcl as text "{hline 13}{c +}{hline 64}";

/* Display difference. */
   local q = invt(`df', `level'/100);
   di in smcl as text %12s "(b-B)" " {c |}" as result
       _col(18) %9.0g `m1'-`m2'
       _col(32) %9.0g `se'
       _col(45) %7.0f
       _col(58) %9.0g (`m1'-`m2')-`q'*(`se')
       _col(70) %9.0g (`m1'-`m2')+`q'*(`se') ;

   di in smcl as text "{hline 13}{c BT}{hline 64}";

   if "`welch'"!="" {;
      local dft "Welch's degrees of freedom = " ;
      local dfc 42;
   };
   else if "`unequal'"!="" {;
      local dft "Satterthwaite's degrees of freedom = " ;
      local dfc 34;
   };
   else {;
      local dft "degrees of freedom = " ;
      local dfc 50;
   };
   ;


/* Display test statistics for Ha */

   di _n as txt "Ho: (b-B) = 0"
        _col(67) "t = " as res %8.4f `t';
   di as txt
        _col(`dfc') as txt "`dft'" as result %8.0g `df';
   di as txt
        _col(55) as txt "Pr(|T| > |t|) = " as result %8.4f `p';


end;
