* this program executes the programs to replicate all results in the paper

set more off

if "`c(os)'" == "Unix" {
   local prefix = ""
   global location = "`prefix'/class"
}
else if "`c(os)'" == "Windows" {
   local prefix = ""
   global location = "`prefix'/class"
}

adopath + "${location}/ado"

* Set globals

	* Paths and Folders
		global folder = "EPR" 
		* To core datasets: 
		global coredata = "$location/data/class_data" 
		* Suffix on core datasets:
		global datasuffix = ""

	* Dates and scenarios 
		* Last historical quarter of macro data: 
		global oldobs = tq(2013q3)
		global scenarios = "cl_base cl_dadv cl_redux"
		* Number of quarters in the horizon
		global horizon = 9                  
		global endobs = $oldobs + $horizon 
		* Starting point for the capgap loop
		global startobs = tq(2002q1)			
		global loopqtr = $oldobs	

	*  Model assuptions 
		* Quarterly (gross) asset growth
		global assetmultiplier = 1.0125
		* Closure rule
		global closure = -999999			

	* Used in 3machine
		* Method used for computing provisions
		global provisions = "tunnel_smooth"    
		* Number of periods over which to smooth
		global prov_adj_period = 8	
		* Payout rate
		global payout = 0.45    
		* Weight for previous dividends
		global divar = 0.90
		* Tax Rate
		global taxrate = 0.350 
		
	* Graphing
		global allgraphs "nodraw graphregion(color(white))"	


/* Purge output files */
local folder "$location/output"
local datafiles: dir "`folder'" files *
foreach datafile of local datafiles {
	if "`datafile'"!="finalized_tables.xlsx" {
		erase "`folder'/`datafile'" 
	}
}
capture {
	local folder "$location/output/figures"
	local datafiles: dir "`folder'" files *
	foreach datafile of local datafiles {
		erase "`folder'/`datafile'" 
	}
}
capture {
	local folder "$location/output/tables"
	local datafiles: dir "`folder'" files *
	foreach datafile of local datafiles {
		erase "`folder'/`datafile'" 
	}
}
capture {
	local folder "$location/output/tables/capital_forecasts"
	local datafiles: dir "`folder'" files *
	foreach datafile of local datafiles {
		erase "`folder'/`datafile'" 
	}
}
cap rmdir "$location/output/figures"
cap rmdir "$location/output/tables/capital_forecasts"
cap rmdir "$location/output/tables"

/* Purge data files */
local folder "$location/data/class_data"
local datafiles: dir "`folder'" files *
foreach datafile of local datafiles {
	if "`datafile'"!="macro_class_wfcasts.dta" & ///
	"`datafile'"!="macro_log.dta" & ///
	"`datafile'"!="capital.dta" & ///
	"`datafile'"!="capital_long.dta" & ///
	"`datafile'"!="class_rawcore.dta" & ///
	"`datafile'"!="core.dta" & ///
	"`datafile'"!="core_wf.dta" {
		erase "`folder'/`datafile'" 
	}
}
capture {
	local folder "$location/data/$folder"
	local datafiles: dir "`folder'" files *
	foreach datafile of local datafiles {
		erase "`folder'/`datafile'" 
	}
}
capture {
	local folder "$location/data/${folder}_concurrentcapgap"
	local datafiles: dir "`folder'" files *
	foreach datafile of local datafiles {
		erase "`folder'/`datafile'" 
	}
}
cap rmdir "$location/data/$folder"
cap rmdir "$location/data/${folder}_concurrentcapgap"

* Create directories
cap mkdir "$location/output/figures"
cap mkdir "$location/output/tables"
cap mkdir "$location/output/tables/capital_forecasts"

cap mkdir "$location/data/$folder"
cap mkdir "$location/data/${folder}_concurrentcapgap"


********************************************************************************
* Run desired subprocesses in order
********************************************************************************


* CLASS MODEL 
do "$location/programs/$folder/1macro.do"
do "$location/programs/$folder/2regs.do"
do "$location/programs/$folder/3machine.do"
do "$location/programs/$folder/4industry.do"
do "$location/programs/$folder/5graphs.do"
do "$location/programs/$folder/6reports.do"
do "$location/programs/$folder/7capgap.do"
do "$location/programs/$folder/8capgapgraphs.do"

* Sensitivities
do "$location/programs/$folder/9SensitivityAssume.do"
do "$location/programs/$folder/10SensitivityAssumeGraphs.do"

* Appendix graphs
do "$location/programs/$folder/appendix_graphs.do"

* Concurrent Capgap
do "$location/programs/${folder}_concurrentcapgap/0doall.do" 
global folder = "EPR"            
do "$location/programs/$folder/figure6.do"

* Compare CLASS projections to performance during financial crisis
do "$location/programs/$folder/CLASS_projections_performance.do" 

* Compare to DFAST
do "$location/programs/$folder/CLASS_DFAST_append_data.do"
do "$location/programs/$folder/CLASS_DFAST_compare.do"

* Asset Share Specifications
do "$location/programs/$folder/asset_comp_macro.do"



