* Create graphs that show sensitivity of CLASS outputs to different assumptions on 
* asset growth, provisions, and dividends

* Retrieve Historical Data
use $location/data/$folder/core_industryonly, clear
keep date cl_tier1_comm_rat
drop if missing(cl_tier1_comm_rat)
rename cl_tier1_comm_rat t1comm
tempfile histdata
save `histdata'

* Cycle through and graph the sensitivity graphs
foreach type in assetmultiplier divar provisiontype {
   use $location/output/tables/industry_data_all_`type', clear
   if "`type'" == "assetmultiplier" {
      replace assetmultiplier = assetmultiplier * 10000
      tostring assetmultiplier, replace force
   }
   else if "`type'" == "divar" {
      replace divar = divar * 10
      tostring divar, replace force
   }
   keep `type' scenario date cl_tier1_comm_rat_fc
   order `type' scenario date cl_tier1_comm_rat_fc
   rename cl_tier1_comm_rat_fc t1comm
   reshape wide t1comm, i(date `type') j(scenario) string
   reshape wide t1comm*, i(date) j(`type') string
   merge 1:1 date using `histdata', nogen
   sort date
   tsset date
   order date t1comm
   drop if date > $endobs
   label var t1comm "Historical"
   if "`type'" == "assetmultiplier" {
      label var t1commcl_base10000 "0.00%"
      label var t1commcl_redux10000 "0.00%"
      label var t1commcl_base10125 "1.25%"
      label var t1commcl_redux10125 "1.25%"
      label var t1commcl_base10250 "2.50%"
      label var t1commcl_redux10250 "2.50%"
      label var t1commcl_base9875 "-1.25%"
      label var t1commcl_redux9875 "-1.25%"
	  
	  local letter = "a"
   }
      else if "`type'" == "provisiontype" {
      label var t1commcl_baseNCO "Provision expense = NCO"
      label var t1commcl_reduxNCO "Provision expense = NCO"
      label var t1commcl_basefour_smooth "ALLL = four quarter rule"
      label var t1commcl_reduxfour_smooth "ALLL = four quarter rule"
      label var t1commcl_basetunnel_smooth "Benchmark CLASS rule"
      label var t1commcl_reduxtunnel_smooth "Benchmark CLASS rule"
	  
	  local letter = "b"
   }
   else if "`type'" == "divar" {
      label var t1commcl_base10 "No Change in Dividends"
      label var t1commcl_redux10 "No Change in Dividends"
      label var t1commcl_base0 "Zero Dividends"
      label var t1commcl_redux0 "Zero Dividends"
      label var t1commcl_base9 "Benchmark CLASS rule, (d=0.9)"
      label var t1commcl_redux9 "Benchmark CLASS rule, (d=0.9)"
	  
	  local letter = "c"
   }
   
   tempfile file`type'
   save `file`type'', replace

   foreach scenario in cl_redux {
      tsline t1comm t1comm`scenario'*, ///
        $allgraphs ///
		lpattern(solid shortdash longdash shortdash_dot longdash_dot) ///
        xtitle("") ///
        ytitle("Tier 1 Common Ratio") ///
        ylab(6(1)12) ///
        legend(symxsize(8) rows(2))
      graph save $location/output/figures/figure_10_`letter', replace
   }
}
