* this program creates transformations and forecasts of macro variables for the CLASS model
set more off
clear all
cd "$location"

local variables = "cpr_t3m cpr_t10y cpr_ur spread d10y stockgrowth growth unchange yyhpi yyhpi_zero yycppi yycppi_zero dbspread dbspread_pos"
local scen_num = wordcount("$scenarios")

* download baseline and stress scenarios for CLASS report, and create class redux scenario
use "$coredata/macro_class_wfcasts.dta", clear

foreach scenario in $scenarios {
   if "`scenario'" != "cl_redux" {
      preserve
      keep if scenario_text == "`scenario'"
      sort date
      tsset date
      gen d10y = cpr_t10y - l1.cpr_t10y
      gen dbspread_pos = dbspread * (dbspread > 0)
      foreach i of varlist `variables' {
         gen f_`i' = `i'
      }
      keep date f_* scenario
      save "$coredata/fcasts_`scenario'", replace
      restore
      }
}

use "$coredata/fcasts_cl_base", replace

* difference between last historical quarter and 2007q2 
local diff = $oldobs - tq(2007q2)
dis "number of quarters from start of crisis redux:  " `diff'

* shifts all the variables back by `diff' periods
foreach i in `variables' {
	gen r_`i' = f_`i'[_n-`diff'] if date > $oldobs & date <= $oldobs + $horizon + 4
	replace f_`i' = f_`i' + (r_`i' - f_`i') if date > $oldobs & date <= $oldobs + $horizon + 4
}

replace scenario = `scen_num'
label define redux_lab `scen_num' "redux"
label values scenario redux_lab
gen scenario_text = "cl_redux"
gen f_cpr_ur_redux = f_cpr_ur if date==$oldobs 
replace f_cpr_ur_redux = f_cpr_ur_redux[_n-1] + ((f_unchange)/4) if date>$oldobs & date<=$oldobs + $horizon + 4
*replacing historical years in redux scenario 
replace f_cpr_ur_redux = f_cpr_ur if date<=$oldobs
replace f_cpr_ur = f_cpr_ur_redux 
drop f_cpr_ur_redux
save "$coredata/fcasts_cl_redux", replace

* Generate historical levels file of relevant data for macrotab
use  "$coredata/macro_class_wfcasts", clear
keep date cpr_realgdp cpr_dow cpr_hpi
drop if date > $oldobs
duplicates drop
sort date
tempfile levels
save `levels', replace

local i = 0
foreach scenario in $scenarios {
	use "$coredata/fcasts_`scenario'", clear
   sort date
   tsset date
   keep date scenario f_cpr_ur f_growth f_stockgrowth f_yyhpi
   merge 1:1 date using `levels'
   drop _merge
	foreach var in realgdp dow hpi {
      gen log_`var' = ln(cpr_`var')
   }
   replace f_growth = f_growth/400
   replace f_stockgrowth = f_stockgrowth/100
   replace f_yyhpi = f_yyhpi/100
   sort date
   replace log_realgdp = l1.log_realgdp + f_growth if missing(log_realgdp)
   replace log_dow = l1.log_dow + f_stockgrowth if missing(log_dow)
   replace log_hpi = l4.log_hpi + f_yyhpi if missing(log_hpi)
	foreach var in realgdp dow hpi {
      replace cpr_`var' = exp(log_`var') if missing(cpr_`var')
   }
   rename f_cpr_ur cpr_ur
   keep date scenario cpr_ur cpr_realgdp cpr_dow cpr_hpi
   foreach var in realgdp dow hpi {
      gen tab_`var' = (cpr_`var'/l3.cpr_`var' - 1)*100
      if "`var'" == "realgdp" | "`var'" == "hpi" {
         replace tab_`var' = tab_`var' * (4/3)
      }
   }
   keep if date == $oldobs | date == $oldobs + 3 | date == $oldobs + 6 | date == $oldobs + 9
   rename cpr_ur tab_ur
   keep date tab*
   gen scenario = "`scenario'"
   order scenario date tab*
   sort date
   if `i' == 1 { 
		append using "$location/output/tables/macrotab.dta"
   }
	local i = 1
	save "$location/output/tables/macrotab.dta", replace
}
replace scenario = "cl_ahistorical" if date == $oldobs
duplicates drop
sort scenario date
replace scenario = substr(scenario,4,.)
cap replace scenario = "historical" if scenario == "ahistorical"
cap replace scenario = "adverse" if scenario == "cadv"
cap replace scenario = "severe stress" if scenario == "dadv"

* Print to file
drop if scenario == "severe stress"
local 1 = "C"
local 2 = "E"
local 3 = "F"
local 4 = "G"
local 5 = "I"
local 6 = "J"
local 7 = "K"
local tab_ur = 5
local tab_realgdp = 6
local tab_dow = 7
local tab_hpi = 8
putexcel set $location/output/finalized_tables.xlsx, modify sheet("Table1")
foreach type in tab_ur tab_realgdp tab_dow tab_hpi {
	forvalues i = 1/7 {
		local number = `type'[`i']
		local formatted_number: display %9.2f `number'
		if `formatted_number' < 0 local formatted_number = "(`formatted_number')"
		qui putexcel ``i''``type'' = `formatted_number'
	}
}
erase "$location/output/tables/macrotab.dta"


