capture restore
clear all
set more off

* Open graphs data
use "$location/output/tables/industry_data_pregraphs.dta", clear

* Drop scenario that is not used
drop *cl_dadv

* Define Labels for selected graphs
foreach variable in cl_nim cl_tradrat cl_nintrat cl_nierat cl_ppnr_ass ///
cl_netinc_assets cl_tier1_comm_rat cl_nco_tot {
	foreach scenario in $scenarios { 
		if "`scenario'" == "cl_base" {
			local scenario1 = "base"
      }
      else if "`scenario'" == "cl_redux" {
         local scenario1 = "crisis redux"
      }
		capture label var `variable'_fc`scenario' "`scenario1'"
   }
	capture label var `variable' "historical"
	label var date	
}

* Graph Tier 1 Common Equity (% of RWA)
preserve
keep cl_tier1_comm_rat_fc* cl_tier1_comm_rat date
save "$location/output/figures/figure_4_a.dta" , replace
restore

sort date
line cl_tier1_comm_rat_fc* cl_tier1_comm_rat date if date >= tq(2001q1), ///
  $allgraphs ///
  lwidth(medthick medthick medthick medthick medthick medthick) ///
  lpattern(shortdash longdash solid) ///
  lcolor(green orange maroon) ///
  ylabel(,labsize(medium)) ///
  xlabel(,labsize(medium)) ///
  xtitle("") ///
  ysc(r(6 13)) ylabel(6(1)13, grid) ///
  legend(cols(4)) 
graph save "$location/output/figures/figure_4_a.gph" , replace

* Graph PPNR components and loan performance

* Figure 2-a
   preserve
   format cl_nim_fc* cl_nim %15.0g
   line cl_nim_fc* cl_nim date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_a.gph" , replace
   keep cl_nim_fc* cl_nim date
   save "$location/output/figures/figure_2_a.dta" , replace
   restore	


* Figure 2-b
  preserve
   format cl_tradrat_fc* cl_tradrat %15.0g
   line cl_tradrat_fc* cl_tradrat date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_b.gph" , replace
   keep cl_tradrat_fc* cl_tradrat date
   save "$location/output/figures/figure_2_b.dta" , replace
   restore	
 
 * Figure 2-c
    preserve
   format cl_nintrat_fc* cl_nintrat %15.0g
   line cl_nintrat_fc* cl_nintrat date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_c.gph" , replace
   keep cl_nintrat_fc* cl_nintrat date
   save "$location/output/figures/figure_2_c.dta" , replace
   restore	
  
 * Figure 2-d
    preserve
   format cl_nierat_fc* cl_nierat %15.0g
   line cl_nierat_fc* cl_nierat date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_d.gph" , replace
   keep cl_nierat_fc* cl_nierat date
   save "$location/output/figures/figure_2_d.dta" , replace
   restore	
 
 * Figure 2-e 
    preserve
   format cl_ppnr_ass_fc* cl_ppnr_ass %15.0g
   line cl_ppnr_ass_fc* cl_ppnr_ass date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_e.gph" , replace
   keep cl_ppnr_ass_fc* cl_ppnr_ass date
   save "$location/output/figures/figure_2_e.dta" , replace
   restore	
 
 * Figure 2-f
    preserve
   format cl_nco_tot_fc* cl_nco_tot %15.0g
   line cl_nco_tot_fc* cl_nco_tot date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_2_f.gph" , replace
   keep cl_nco_tot_fc* cl_nco_tot date
   save "$location/output/figures/figure_2_f.dta" , replace
   restore	

* Figure 3
   preserve
   format cl_netinc_assets_fc* cl_netinc_assets %15.0g
   line cl_netinc_assets_fc* cl_netinc_assets date, ///
     $allgraphs ///
     legend(size(medsmall) cols(4)) ///
	 lpattern(shortdash longdash solid) ///
     ylabel(,labsize(medium)) ///
     xlabel(,labsize(medium)) ///
     xtitle("") ///
     lwidth(medthick medthick medthick medthick medthick medthick) ///
     lcolor(green orange maroon) ylab(#8, angle(0))
   graph save "$location/output/figures/figure_3.gph" , replace
   keep cl_netinc_assets_fc* cl_netinc_assets date
   save "$location/output/figures/figure_3.dta" , replace
   restore	

