clear all
set more off

local first = 0

foreach scenario in $scenarios {
   use "$location/output/tables/capital_forecasts_`scenario'.dta", clear
   * Drop aggregated categories of institutions & a few specific problem entities
   drop if (entity >9990001 | entity == 52719 | entity == 3822016)

   * Calculate period in forecast horizon when industry Tier 1 common is minimized
   preserve
   collapse (sum) tier1_comm_fc rwa_fc, by(date)
   gen tier1_comm_rat_fc = tier1_comm_fc / rwa_fc * 100
   keep if date > $oldobs & date <= $oldobs + $horizon
   sort tier1_comm_rat_fc
   local nadir_date = date[1]
   dis "Capital minimum reached on: `nadir_date'"
   restore

   gen tier1_comm_rat_fc = tier1_comm_fc / rwa_fc * 100

   keep if date == `nadir_date'
		
   * Save capital distribution data for graph
   preserve
   gen tier1_rat_`scenario' = tier1_comm_rat_fc
   sort tier1_rat_`scenario'
   gen cumassetshare_`scenario' = sum(assetshare)
   qui summ cumassetshare_`scenario'
   replace cumassetshare_`scenario' = cumassetshare_`scenario' * (100/r(max))
   gen cumrwashare_`scenario' = sum(rwashare)
   qui summ cumrwashare_`scenario'
   replace cumrwashare_`scenario' = cumrwashare_`scenario' * (100/r(max))
   label var tier1_comm_rat_fc "Capital ratio [Tier 1 common / RWA]"
   label var cumassetshare_`scenario' "`scenario'"
   label var cumrwashare_`scenario' "`scenario'"
	
   keep tier1_rat_`scenario' cumassetshare cumrwashare
	if `first' == 1 {
      append using "$location/output/figures/capdist_graph_all.dta"
   }
   save "$location/output/figures/capdist_graph_all.dta", replace
   restore
	
   * Create statistics for capital gap table
   preserve
   forvalues i = 3/8 {
		gen gap`i' = 0
		replace gap`i' = (`i' - tier1_comm_rat_fc) / 100 * (rwa_fc / 1000000) if tier1_comm_rat_fc <= `i'
		gen gap_share`i' = (gap`i' > 0) * assetshare
		gen gap_rwashare`i' = (gap`i' > 0) * rwashare
		gen gap_yes`i' = (gap`i' > 0 & entity ~= 9990001)
		gen gap_topseven`i' = (cl_assets > 700000000 & entity ~= 9990001) * gap`i'
		gen gap_100bn`i' = (cl_assets <= 700000000 & cl_assets > 100000000 & entity ~= 9990001) * gap`i'
		gen gap_10bn`i' = (cl_assets <= 100000000 & cl_assets > 10000000 & entity ~= 9990001) * gap`i'
		gen gap_small`i' = (cl_assets <= 10000000 | entity == 9990001) * gap`i'   
   }

   keep date cl_assets rwa_fc tier1_comm_rat_fc entity gap*
   gen scenario = "`scenario'"
   if `first' == 1 {
      append using "$location/output/tables/capgap_all.dta"
   }
   save "$location/output/tables/capgap_all.dta", replace
   local first = 1
   restore	
}

* Create graph of cumulative capital distribution
use "$location/output/figures/capdist_graph_all.dta", clear

* Eliminate unused scenario
drop *cl_dadv

foreach scenario in $scenarios { 
   if "`scenario'" == "cl_base" {
      local scenario1 = "base"
   }
   else if "`scenario'" == "cl_redux" {
      local scenario1 = "crisis redux"
   }

   capture label var cumassetshare_`scenario' "`scenario1'"
}

preserve
keep cumassetshare_* tier1_rat_* 
qui save "$location/output/figures/figure_4_b.dta", replace
restore

* Create graphing locals
local colors = "green orange"
local patterns = "shortdash longdash"
local dist_cmd " "
local scencount = wordcount("cl_base cl_redux")

forvalues i = 1/`scencount' {
   local scen = word("cl_base cl_redux",`i')
   local scen_color = word("`colors'",`i')
   local scen_pattern = word("`patterns'",`i')
   local scen_cmd = " (line cumassetshare_`scen' tier1_rat_`scen' if tier1_rat_`scen' > 0 & tier1_rat_`scen' < 15, lwidth(medthick) lcolor(`scen_color') lpattern(`scen_pattern'))"
   local dist_cmd `dist_cmd' `scen_cmd'
}
twoway `dist_cmd', ///
  $allgraphs ///
  ytitle("% industry assets") ///
  legend(cols(4))
graph save "$location/output/figures/figure_4_b.gph", replace

* Create table used in generation of capital gap graph
use "$location/output/tables/capgap_all.dta", clear

collapse (sum) gap*, by(scenario)
reshape long gap gap_share gap_rwashare gap_yes gap_topseven gap_100bn gap_10bn gap_small, i(scenario) j(target)
keep scenario target gap gap_share gap_rwashare gap_yes gap_topseven gap_100bn gap_10bn gap_small

preserve
keep scenario target gap gap_share gap_rwashare gap_yes
save "$location/output/tables/capgap_full", replace
restore

