* Quarter local
local q 2013q3

* Open up severely adverse projections
use "$location/output/tables/capital_forecasts_cl_dadv", clear
gsort -entity date

* Keep CCAR Firms
merge m:1 entity using $location/data/raw/CCAR_Data, keepusing(entity) keep(3) nogen

* Drop Non-Forecast Period
gen forecast = (date != tq(`q'))

* Source variable
gen source = "CLASS"

* Keep asset projections
drop assets
rename cl_assets assets

* Fix units (thousands -> millions)
foreach var of varlist cl_ppnr_fc qllprov_fc pretax_fc tier1_comm_fc tier1_rbc_fc assets rwa qnetxoff_* ln_* {
   replace `var' = `var'/1000
}

* Generate "last" and "minimum" values for capital ratios
gen min_tier1_ratSup = cl_tier1_rat_fc 
gen min_t1c_ratSup = cl_tier1_comm_rat_fc

gen min_tier1_ratAdj = cl_tier1_rat_fc
gen min_t1c_ratAdj = cl_tier1_comm_rat_fc

gen min_tier1_ratDFS = cl_tier1_rat_fc 
gen min_t1c_ratDFS = cl_tier1_comm_rat_fc

sort entity date

* Collapse to historical and forecast periods 
collapse ///
  (sum) ///
  cl_ppnr_fc ///
  qllprov_fc ///
  pretax_fc ///
  qnetxoff_firstlien_fc ///
  qnetxoff_jrlien_fc ///
  qnetxoff_ci_fc ///
  qnetxoff_cre_fc ///
  qnetxoff_cc_fc ///
  qnetxoff_othcons_fc ///
  qnetxoff_tot_fc ///
  (mean) ///
  ln_firstlien ///
  ln_jrlien ///
  ln_ci ///
  ln_cre ///
  ln_cc ///
  ln_othcons ///
  ln_tot ///
  assets ///
  rwa ///
  (last) ///	
  cl_tier1_rat_fc ///
  cl_tier1_comm_rat_fc ///
  (min) ///
  min_tier1_ratSup ///
  min_t1c_ratSup ///
  min_tier1_ratDFS ///
  min_t1c_ratDFS ///
  min_tier1_ratAdj ///
  min_t1c_ratAdj ///
  , by(source entity name forecast)


order source ///
  entity ///
  name ///
  forecast ///
  cl_tier1_rat_fc ///
  cl_tier1_comm_rat_fc ///
  min_tier1_ratSup ///
  min_t1c_ratSup ///
  min_tier1_ratDFS ///
  min_t1c_ratDFS ///
  min_tier1_ratAdj ///
  min_t1c_ratAdj ///
  cl_ppnr_fc ///
  qllprov_fc ///
  pretax_fc ///
  assets ///
  rwa ///
  qnetxoff_firstlien_fc ///
  qnetxoff_jrlien_fc ///
  qnetxoff_ci_fc ///
  qnetxoff_cre_fc ///
  qnetxoff_cc_fc ///
  qnetxoff_othcons_fc ///
  qnetxoff_tot_fc ///
  ln_firstlien ///
  ln_jrlien ///
  ln_ci ///
  ln_cre ///
  ln_cc ///
  ln_othcons ///
  ln_tot

* Make 0's missing
foreach var of varlist cl_tier1_rat_fc - ln_tot {
   replace `var' = . if `var' == 0
}

* Generate NCO Rates
foreach ncotype in firstlien jrlien ci cre cc othcons tot {
   gen nco_`ncotype'_rat = 100*(qnetxoff_`ncotype'_fc/ln_`ncotype')
   rename qnetxoff_`ncotype'_fc nco_`ncotype'_amt
   if "`ncotype'" != "tot" {
      drop ln_`ncotype'
   }
}

* Rename variables
rename cl_tier1_rat_fc tier1_rat
rename cl_tier1_comm_rat_fc t1c_rat

* Generate PPNR Rates
gen ppnr_rat = (cl_ppnr_fc/assets)*100
rename cl_ppnr_fc ppnr_amt

* Generate Provisions Data
gen provisions_rat = (qllprov_fc/ln_tot)*100
label var provisions_rat "Provision Ratio over Loans"
rename qllprov_fc provisions_amt
rename ln_tot loans

* Generate Pretax Rates
gen pretax_rat = (pretax_fc/assets)*100
rename pretax_fc pretax_amt

* Generate difference between last projected capital ratio and the last historical
* Generate difference between the minimum projected capital ratio and the last historical
sort entity forecast
foreach rat in t1c tier1 {
   foreach type in DFS Sup Adj {
      by entity: gen diff_end_`rat'`type' = `rat'_rat[2] - `rat'_rat[1]
      label var diff_end_`rat'`type' "Difference between 2015Q4 `rat' `type' and 2013Q3 `rat'"

      by entity: gen diff_min_`rat'`type' = min_`rat'_rat`type'[2] - `rat'_rat[1]
      label var diff_min_`rat'`type' "Difference between min `rat' `type' and 2013Q3 `rat'"
   }
}

* Create capital ratio variable for DFS, Sup, and Adj 
foreach rat in t1c tier1 {
   foreach type in DFS Sup Adj {
      gen `rat'_rat`type' = `rat'_rat
   }
}
drop t1c_rat tier1_rat

* Keep only forecast period
keep if forecast
drop forecast
tempfile class
save `class', replace

* Append CLASS and CCAR data to compare
use $location/data/raw/CCAR_Data, clear
append using `class'
spread name, by(entity)
gen provass_rat = provisions_rat * loans/assets
gen other_rat = provass_rat + pretax_rat - ppnr_rat
label var other_rat "Other Ratio over Assets"
replace date = tq(2015q4) if missing(date)

* Make local of variables we want to compare
local comparevars ///
  ppnr_amt ///
  ppnr_rat ///
  provisions_amt ///
  provisions_rat ///
  provass_rat ///
  other_rat ///
  pretax_amt ///
  pretax_rat ///
  tier1_ratDFS ///
  tier1_ratSup ///
  tier1_ratAdj ///
  t1c_ratDFS ///
  t1c_ratSup ///
  t1c_ratAdj ///
  min_t1c_ratSup ///
  min_tier1_ratSup ///
  min_t1c_ratDFS ///
  min_tier1_ratDFS ///
  min_t1c_ratAdj ///
  min_tier1_ratAdj ///
  nco_firstlien_amt ///
  nco_firstlien_rat ///
  nco_jrlien_amt ///
  nco_jrlien_rat ///
  nco_ci_amt ///
  nco_ci_rat ///
  nco_cre_amt ///
  nco_cre_rat ///
  nco_cc_amt ///
  nco_cc_rat ///
  nco_othcons_amt ///
  nco_othcons_rat ///
  nco_tot_amt ///
  nco_tot_rat ///
  diff_min_t1cDFS ///
  diff_min_t1cSup ///
  diff_min_t1cAdj ///
  diff_min_tier1DFS ///
  diff_min_tier1Sup ///
  diff_min_tier1Adj ///
  diff_end_tier1DFS ///
  diff_end_tier1Sup ///
  diff_end_tier1Adj ///
  diff_end_t1cDFS ///
  diff_end_t1cSup ///
  diff_end_t1cAdj ///
  assets ///
  loans

order entity name date source `comparevars'

* Save
save "$location/output/tables/CCAR_CLASS_EPR_GraphData", replace
