* Quarter local
local q 2013q3

* Retrieve last historical RWA
use "$location/output/tables/capital_forecasts_cl_dadv", clear
keep date entity name rwa
rename rwa rwa_last_hist
merge m:1 entity using $location/data/raw/CCAR_Data, keepusing(entity) keep(3) nogen
sort date
local last_historical = date[1]
keep if date == `last_historical'
tempfile historical_rwa
save `historical_rwa', replace


* Calculate difference in industry tier 1 common over forecast horizon in CLASS
use "$location/output/tables/capital_forecasts_cl_dadv", clear
order date entity name rwa tier1_comm_fc
keep date entity name rwa tier1_comm_fc
merge m:1 entity using $location/data/raw/CCAR_Data, keepusing(entity) keep(3) nogen
sort entity date
collapse (first) first_t1c=tier1_comm_fc first_rwa=rwa (last) last_t1c=tier1_comm_fc last_rwa=rwa, by(entity name)
collapse (sum) industry_first_t1c=first_t1c industry_first_rwa=first_rwa industry_last_t1c=last_t1c industry_last_rwa=last_rwa
gen industry_t1c_rat_beg = industry_first_t1c/industry_first_rwa * 100
gen industry_t1c_rat_end = industry_last_t1c/industry_last_rwa * 100
gen diff_end_industry_t1c_rat = industry_t1c_rat_end - industry_t1c_rat_beg
gen source = "CLASS"
keep source diff_end_industry_t1c_rat
order source diff_end_industry_t1c_rat
tempfile CLASS_diff_industry_t1c_rat
save `CLASS_diff_industry_t1c_rat', replace

* Calculate difference in industry tier 1 common over forecast horizon in CCAR and append from CLASS
use "$location/output/tables/CCAR_CLASS_EPR_GraphData", clear
drop rwa
merge m:1 entity using `historical_rwa', keepusing(rwa_last_hist) assert(3) nogen
keep if source == "CCAR"
collapse (mean) diff_end_industry_t1c_rat=diff_end_t1cDFS [aw=rwa_last_hist], by(source)
append using `CLASS_diff_industry_t1c_rat'
tempfile diff_industry_t1c_rat
save `diff_industry_t1c_rat', replace

* Aggregate to industry
use "$location/output/tables/CCAR_CLASS_EPR_GraphData", clear
collapse (sum) ppnr_amt provisions_amt pretax_amt assets loans, by(source)
gen ppnr_rat = 100*ppnr_amt/assets
gen provisions_rat = 100*provisions_amt/assets
gen lprovisions_rat = 100*provisions_amt/loans
gen pretax_rat = 100*pretax_amt/assets
gen other_rat = provisions_rat + pretax_rat - ppnr_rat
keep source *_rat
order source ppnr* provisions* other* pretax*
merge 1:1 source using `diff_industry_t1c_rat', assert(3) nogen

* Print to file
gsort -source
putexcel set $location/output/finalized_tables.xlsx, modify sheet("Table7")
local ppnr_rat = 5
local provisions_rat = 6
local other_rat = 7
local pretax_rat = 8
local diff_end_industry_t1c_rat = 9

foreach var in ppnr_rat provisions_rat other_rat pretax_rat diff_end_industry_t1c_rat {
	local class : display %9.2f `=`var'[1]'
	local dfast : display %9.2f `=`var'[2]'
	local difference : display %9.2f `=`var'[1]-`var'[2]'
	* CLASS
	putexcel C``var'' = "`class'"
	* DFAST
	putexcel D``var'' = "`dfast'"
	* Difference
	putexcel E``var'' = "`difference'"
}


* Get Relevant slope coefficients & r^2 values
use "$location/output/tables/CCAR_CLASS_EPR_GraphData", clear

* Reshape dataset
local vars "ppnr_rat provisions_rat pretax_rat other_rat diff_end_t1cDFS"
keep entity name source `vars'
reshape wide `vars', i(entity name) j(source) string

* Regressions and print to file
local coeff_column = "G"
local r_column = "H"
foreach var in `vars' {
   reg `var'CLASS `var'CCAR
   
      print_info `var'CLASS `var'CCAR 3 table7 _b[`var'CCAR] _se[`var'CCAR]
      putexcel `coeff_column'`r(excel_row)' = "`r(formatted_coeff)'`r(stars)'"
      local r_squared : display %9.3f e(r2)
      putexcel `r_column'`r(excel_row)' = "`r_squared'"

}
   
   
