* Set useful globals and locals
local maindate "2007q2"
local nummaindate = tq(`maindate')
local keepdate "2009q3"
local keepvars_historical "cl_tier1_comm_rat cl_netinc_assets cl_assets qnetinc tier1_comm rwa cl_ppnr qnetxoff_tot ln_tot cl_nco_tot cl_ppnr_ass "
local keepvars_forecast : subinstr local keepvars_historical " " "_fc ", all
local ordervars : subinstr local keepvars_historical " " "* ", all

* Open up the 9-quarter forecasts as of 2007Q2 and merge-in actual realizations over the same period
	use "$location/output/tables/capital_forecasts/capital_forecasts_cl_redux_q`nummaindate'", clear
	replace date = date - (tq(2013q3) - tq(`maindate'))
	gen cl_netinc_assets_fc = (qnetinc_fc / cl_assets) * 400
	drop if entity >= 9990001
	rename cl_assets cl_assets_fc
	rename ln_tot ln_tot_fc
	rename cl_nco_tot cl_nco_tot_fc 
	keep date entity name `keepvars_forecast'

	preserve
	* Retrieve actuals
	use "$location/data/class_data/class_rawcore", clear
	drop if entity >= 9990001
	gen cl_tier1_comm_rat = (tier1_comm / rwa) * 100
	gen cl_ppnr = qint_inc_net + qnonint_inc - qnonint_exp
	rename assets cl_assets
	gen cl_ppnr_ass = cl_ppnr / cl_assets * 400
	gen cl_nco_tot = 400 * (qnetxoff_tot / ln_tot)
	gen cl_netinc_assets = (qnetinc / cl_assets) * 400
	keep date entity `keepvars_historical'
	keep if date >=tq(`maindate') & date < tq(`maindate') + 10
	tempfile relevantcore
	save `relevantcore', replace
	restore

	merge 1:1 entity date using `relevantcore', keep(1 3) nogen
	order date entity name `ordervars'
	save "$location/output/tables/ComparisonData", replace

* Find subset of firms that exist throughout the entire 9-quarter forecast horizon 
	use "$location/output/tables/ComparisonData", clear
	keep if date <= tq(`keepdate') & date >= tq(`maindate')
	local diff = tq(`keepdate') - tq(`maindate') + 1
	keep if !missing(cl_tier1_comm_rat)
	sort entity date
	keep date entity name
	gen num = 1
	collapse (sum) num, by(entity)
	keep if num == `diff'
	keep entity
	count
	tempfile q`keepdate'entities
	save `q`keepdate'entities', replace
	
* Find date where industry tier 1 common ratio is minimized
* "Industry" is composed of only firms that exist throughout the entire 9-quarter forecast horizon 
	use "$location/data/class_data/class_rawcore", clear
	merge m:1 entity using `q`keepdate'entities', keep(3) nogen
	collapse (sum) tier1_comm rwa, by(date)
	gen cl_tier1_comm_rat = (tier1_comm / rwa) * 100
	keep if date >= tq(`maindate') & date <= tq(`keepdate')
	sort cl_tier1_comm_rat
	local nadir_date = date[1]
	local nadir_date: di %tq `nadir_date'
	di "Nadir date is `nadir_date'"

* Calculate change in tier 1 common ratio from 2007Q2 to `nadir_date', by entity
* Only entities that exist throughout the entire 9-quarter forecast horizon are included
	use "$location/output/tables/ComparisonData", clear
	merge m:1 entity using `q`keepdate'entities', keep(3) nogen
	keep if date >= tq(`maindate') & date <= tq(`nadir_date')
	keep entity date tier1_comm_fc tier1_comm rwa_fc rwa
	sort entity date
	
	collapse (first) start_t1c_fc=tier1_comm_fc start_t1c=tier1_comm start_rwa_fc=rwa_fc ///
	start_rwa=rwa (last) end_t1c_fc=tier1_comm_fc end_t1c=tier1_comm end_rwa_fc=rwa_fc ///
	end_rwa=rwa, by(entity)

	gen d_tier1_comm_rat_fc = 100 * (end_t1c_fc/end_rwa_fc - start_t1c_fc/start_rwa_fc)
	label var d_tier1_comm_rat_fc "Predicted T1C ratio (`maindate' - `nadir_date')"
	gen d_tier1_comm_rat = 100 * (end_t1c/end_rwa - start_t1c/start_rwa)
	label var d_tier1_comm_rat "Actual T1C ratio (`maindate' - `nadir_date')"

	keep entity d_* start_t1c_fc start_rwa_fc end_t1c_fc end_rwa_fc start_t1c start_rwa end_t1c end_rwa
	order entity d_* start_t1c_fc start_rwa_fc end_t1c_fc end_rwa_fc start_t1c start_rwa end_t1c end_rwa

	tempfile difference_entity_t1c
	save `difference_entity_t1c', replace
	
	* Calculate change in industry tier 1 common ratio from 2007Q2 to when `nadir_date'
	* "Industry" is composed of only firms that exist throughout the entire 9-quarter forecast horizon 
	preserve
	collapse (sum) start_t1c_fc start_rwa_fc end_t1c_fc end_rwa_fc start_t1c start_rwa end_t1c end_rwa
	
	gen d_tier1_comm_rat = 100 * (end_t1c/end_rwa - start_t1c/start_rwa)
	gen d_tier1_comm_rat_fc = 100 * (end_t1c_fc/end_rwa_fc - start_t1c_fc/start_rwa_fc)
	gen name = "Industry Aggregate during Crisis"
	keep name d_*
	tempfile difference_t1c
	save `difference_t1c', replace
	restore

* Aggregate variables over forecast horizon, by entity
* Only entities that exist throughout the entire 9-quarter forecast horizon are included
	use "$location/output/tables/ComparisonData", clear
	merge m:1 entity using `q`keepdate'entities', keep(3) nogen
	keep if date <= tq(`keepdate') & date > tq(`maindate')
	sort entity date
	collapse (sum) qnetinc_fc qnetinc cl_assets_fc cl_assets ln_tot_fc ln_tot qnetxoff_tot_fc qnetxoff_tot cl_ppnr_fc cl_ppnr, by(entity)

		* Industry aggregates over forecast horizon
		* "Industry" is composed of only firms that exist throughout the entire 9-quarter forecast horizon 
		putexcel set $location/output/finalized_tables.xlsx, modify sheet("Table8")
		preserve
		collapse (sum) qnetinc_fc qnetinc cl_assets_fc cl_assets ln_tot_fc ln_tot qnetxoff_tot_fc qnetxoff_tot cl_ppnr_fc cl_ppnr

		gen cl_netinc_assets_fc = (qnetinc_fc / cl_assets_fc) * 400
		gen cl_ppnr_ass_fc = cl_ppnr_fc / cl_assets_fc * 400
		gen cl_nco_tot_fc = 400 * (qnetxoff_tot_fc / ln_tot_fc)
		
		gen cl_netinc_assets  = (qnetinc / cl_assets) * 400
		gen cl_ppnr_ass = cl_ppnr / cl_assets * 400
		gen cl_nco_tot = 400 * (qnetxoff_tot / ln_tot)
		
		keep cl_netinc_assets* cl_ppnr_ass* cl_nco_tot*
		order cl_netinc_assets* cl_ppnr_ass* cl_nco_tot*
		
		gen name = "Industry Aggregate during Crisis"
		order name

		merge 1:1 name using `difference_t1c', assert(3) nogen

		* Print to excel file
		local cl_netinc_assets = 11
		local cl_nco_tot = 10
		local cl_ppnr_ass = 9
		local d_tier1_comm_rat = 13

		foreach var in cl_netinc_assets cl_nco_tot cl_ppnr_ass d_tier1_comm_rat {
			local model : display %9.2f `=`var'_fc[1]'
			local actual : display %9.2f `=`var'[1]'
			local difference : display %9.2f `=`var'_fc[1]-`var'[1]'
			* Model
			putexcel C``var'' = "`model'"
			* Actual
			putexcel D``var'' = "`actual'"
			* Difference
			putexcel E``var'' = "`difference'"
		}

		save "$location/output/tables/Industry_aggregatecrisis", replace
		export excel using "$location/output/tables/Industry_aggregatecrisis", firstrow(var) replace
		restore

	gen cl_netinc_assets_fc = (qnetinc_fc / cl_assets_fc) * 400
	gen cl_ppnr_ass_fc = cl_ppnr_fc / cl_assets_fc * 400
	gen cl_nco_tot_fc = 400 * (qnetxoff_tot_fc / ln_tot_fc)
	
	gen cl_netinc_assets  = (qnetinc / cl_assets) * 400
	gen cl_ppnr_ass = cl_ppnr / cl_assets * 400
	gen cl_nco_tot = 400 * (qnetxoff_tot / ln_tot)

	merge 1:1 entity using `difference_entity_t1c', assert(3) nogen

	keep entity cl_netinc_assets* cl_ppnr_ass* cl_nco_tot* d_* start* end* cl_assets 
	order entity cl_netinc_assets* cl_ppnr_ass* cl_nco_tot* d_* start* end* cl_assets 

	save  "$location/output/tables/ComparisonData_survivors_regs_data", replace
	export excel using "$location/output/tables/ComparisonData_survivors_regs_data", firstrow(var) replace

	* Perform regressions and print to excel file
	forvalues i = 1/2 {

	   if "`i'" == "1" {
		  local assetweight = ""
		  local aw = ""
		  local coeff_column = "J"
		  local r_column = "K"
	   }
	   else if "`i'" == "2" {
		  local assetweight = "[aw=cl_assets]"
		  local aw = "Asset-Weighted"
		  local coeff_column = "G"
		  local r_column = "H"
	   }
	   
	   preserve
	   foreach var in cl_netinc_assets d_tier1_comm_rat cl_ppnr_ass cl_nco_tot {
		  _pctile `var', p(98)
		  local top98 = `r(r1)'
		  _pctile `var', p(2)
		  local top2 = `r(r1)'
		  
		  replace `var' = `top98' if `var' >= `top98'
		  replace `var' = `top2' if `var' <= `top2'
		  
		  reg `var' `var'_fc `assetweight'
		  print_info `var' `var'_fc 3 table8 _b[`var'_fc] _se[`var'_fc]
		  putexcel `coeff_column'`r(excel_row)' = "`r(formatted_coeff)'`r(stars)'"
		  local r_squared : display %9.3f e(r2)
		  putexcel `r_column'`r(excel_row)' = "`r_squared'"
	   }
	   restore
	}




