*
* This program collapses model output from machine.do into industry ratios, saves in a new dataset, and creates graphs
*
clear all
set more off
cd "$location"

local first = 0

* set to value of 1 if you want to keep just the ccar / irr banks, 0 otherwise
foreach scenario in $scenarios {
   
	use "output/tables/capital_forecasts_`scenario'_point.dta", clear
	drop if entity == 9991000
	if $ccar_on == 1 {
		keep if ccar == 1
   }
	else if $liscc_on == 1 {
		keep if liscc == 1
   }
   else if $cappr_on == 1 {
      keep if cappr == 1
   }
   else if $qs_on == 1 {
      keep if qs == 1
   }
   else {
      keep if entity <= 9990001
   }
	preserve
	gen countbank=1 

	* Collapse revenue and capital variables, and create industry ratios
	
	collapse (sum) qint_inc_net_fc cl_intearn_ass qtradrev_inc_fc cl_trading_ass qnonint_inc_fc qnonint_inc_nt_fc cl_assets liab_tot_fc qnonint_exp_fc qnonint_exp_comp_fc qnonint_exp_fass_fc qnonint_exp_allother_fc cl_ppnr_fc qllprov_fc llres_fc qnetxoff_tot_fc fourqtr_nco_tot_fc qafssecur_inc_fc qhtmsecur_inc_fc afssec qtaxes_inc_fc qminor_int_fc qextra_inc_fc qnetinc_fc cl_dividend_fc equity_fc t1_regcap_subtot_fc dta_dis_tot_fc tier1_rbc_fc tier1_comm_fc rwa_fc pretax_fc countbank, by(date)

	sort date

	* list for top down PPNR series:
   
	* list for top down NCO series: each with cl_nco_`i'_fc
	* rre heloc firstlien jrlien cre const multi nfnr othre cc othcons ci lease allother fgovt oth agr dep tot

	gen cl_nim_fc = (qint_inc_net_fc / cl_intearn_ass) * 400
	gen cl_tradrat_fc = (qtradrev_inc_fc / cl_trading_ass) * 400
	gen cl_nintrat_fc = (qnonint_inc_nt_fc / cl_assets) * 400

	gen cl_netii_rwa_fc = (qint_inc_net_fc / rwa_fc) * 400
	gen cl_nii_tot_rwa_fc = ((qtradrev_inc_fc + qnonint_inc_nt_fc) / rwa_fc) * 400
	gen cl_nii_tr_rwa_fc = (qtradrev_inc_fc / rwa_fc) * 400
	gen cl_nii_ntr_rwa_fc = (qnonint_inc_nt_fc / rwa_fc) * 400
   
	gen cl_netii_fc = (qint_inc_net_fc / cl_assets) * 400
	gen cl_nii_tot_fc = ((qtradrev_inc_fc + qnonint_inc_nt_fc) / cl_assets) * 400
	gen cl_nii_tr_fc = (qtradrev_inc_fc / cl_assets) * 400
	gen cl_nii_ntr_fc = (qnonint_inc_nt_fc / cl_assets) * 400
   
   gen cl_nierat_fc = (qnonint_exp_fc / cl_assets) * 400
   gen cl_nie_tot_rwa_fc = (qnonint_exp_fc / rwa_fc) * 400
   gen cl_nie_comp_fc = (qnonint_exp_comp_fc / cl_assets) * 400
   gen cl_nie_comp_rwa_fc = (qnonint_exp_comp_fc / rwa_fc) * 400
   gen cl_nie_fass_fc = (qnonint_exp_fass_fc / cl_assets) * 400
   gen cl_nie_fass_rwa_fc = (qnonint_exp_fass_fc / rwa_fc) * 400
   gen cl_nie_allother_fc = (qnonint_exp_allother_fc / cl_assets) * 400
   gen cl_nie_allother_rwa_fc = (qnonint_exp_allother_fc / rwa_fc) * 400
	
	gen cl_ppnr_ass_fc = (cl_ppnr_fc / cl_assets) * 400
	gen cl_ppnr_rwa_fc = (cl_ppnr_fc / rwa_fc) * 400
	gen cl_qnetxoff_tot_fc = qnetxoff_tot_fc
	
	gen cl_qllprov_rwa_fc = (qllprov_fc / rwa_fc) * 400
	
	gen cl_afs_return_fc = (qafssecur_inc_fc / afssec) * 400
   
	gen cl_pretax_rwa_fc = (pretax_fc / rwa_fc) * 400
	
	gen cl_payout_rat_fc = (cl_dividend_fc / qnetinc_fc) * 100
	replace cl_payout_rat_fc = 100 if ( cl_payout_rat_fc > 100 | qnetinc_fc < 0 )
	gen cl_netinc_assets_fc = (qnetinc_fc / cl_assets) * 400
	gen cl_netinc_rwa_fc = (qnetinc_fc / rwa_fc) * 400
	gen cl_dividend_rwa_fc = (cl_dividend_fc / rwa_fc) * 400

	gen cl_equity_rat_fc = (equity_fc / rwa_fc) * 100

	gen cl_tier1_comm_rat_fc = (tier1_comm_fc / rwa_fc) * 100
	gen cl_tier1_rat_fc = (tier1_rbc_fc / rwa_fc) * 100
	gen cl_assets_fc = cl_assets
	gen cl_liab_tot_fc = liab_tot_fc
	
	summarize cl_tier1_rat_fc
	
   sort date
   gen scenario = "`scenario'"
   save "$location/output/tables/industry_data_`scenario'_point.dta", replace
	restore
	
   * Collapse net chargeoff variables, and create industry ratios
   
	foreach i in rre heloc firstlien jrlien cre const multi nfnr othre cc othcons ci lease allother fgovt oth agr dep tot {
      preserve
      collapse (sum) qnetxoff_`i'_fc ln_`i', by(date)
      rename ln_`i' ln_`i'_fc
      * Note: I multiply by four here because the chargeoff rates are annualized
      gen cl_nco_`i'_fc = 4 * qnetxoff_`i'_fc * 100 / ln_`i'
      keep date cl_nco_`i'_fc qnetxoff_`i'_fc ln_`i'_fc
      sort date
      qui merge 1:1 date using "output/tables/industry_data_`scenario'_point.dta"
      qui drop _merge
      save "output/tables/industry_data_`scenario'_point.dta", replace
      restore
   }
   
   * Merge scenarios together
   use "$location/output/tables/industry_data_`scenario'_point.dta", clear
   if `first' == 1 {
      append using "output/tables/industry_data_all_point.dta"
   }
   save "$location/output/tables/industry_data_all_point.dta", replace
	local first = 1
}

keep date tier1_*_fc cl_*_fc qllprov_fc llres_fc qnetxoff_tot_fc scenario
reshape wide tier1_*_fc cl_*_fc qllprov_fc llres_fc qnetxoff_tot_fc, i(date) j(scenario) string
capture drop _merge

sort date
qui merge 1:1 date using "data/$folder/core_industryonly.dta"
qui drop _merge

save "$location/output/tables/industry_data_pregraphs_point.dta", replace
