clear all
set more off

local first = 0

foreach scenario in $scenarios {
	use "$location/output/tables/capital_forecasts_`scenario'_point.dta", clear
	* Drop aggregated categories of institutions & a few specific problem entities
	drop if (entity >9990001 | entity == 52719 | entity == 3822016)

   * Calculate period in forecast horizon when industry Tier 1 common is minimized
   preserve
   collapse (sum) tier1_comm_fc rwa_fc, by(date)
   gen tier1_comm_rat_fc = tier1_comm_fc / rwa_fc * 100
   keep if date > $oldobs & date <= $oldobs + $horizon
   sort tier1_comm_rat_fc
   local nadir_date = date[1]
   dis "Capital minimum reached on: `nadir_date'"
   restore

   gen tier1_comm_rat_fc = tier1_comm_fc / rwa_fc * 100

   keep if date == `nadir_date'

   * Create statistics for capital gap table
   forvalues i = 3/8 {
		gen gap`i' = 0
		replace gap`i' = (`i' - tier1_comm_rat_fc) / 100 * (rwa_fc / 1000000) if tier1_comm_rat_fc <= `i'
		gen gap_share`i' = (gap`i' > 0) * assetshare
		gen gap_rwashare`i' = (gap`i' > 0) * rwashare
		gen gap_yes`i' = (gap`i' > 0 & entity ~= 9990001)
		gen gap_topseven`i' = (cl_assets > 700000000 & entity ~= 9990001) * gap`i'
		gen gap_100bn`i' = (cl_assets <= 700000000 & cl_assets > 100000000 & entity ~= 9990001) * gap`i'
		gen gap_10bn`i' = (cl_assets <= 100000000 & cl_assets > 10000000 & entity ~= 9990001) * gap`i'
		gen gap_small`i' = (cl_assets <= 10000000 | entity == 9990001) * gap`i'   
   }

   keep date cl_assets rwa_fc tier1_comm_rat_fc entity gap*
   gen scenario = "`scenario'"
   if `first' == 1 {
      append using "$location/output/tables/capgap_all_point.dta"
   }
   save "$location/output/tables/capgap_all_point.dta", replace
   local first = 1
}

* Create table used in generation of capital gap graph
use "$location/output/tables/capgap_all_point.dta", clear

collapse (sum) gap*, by(scenario)
reshape long gap gap_share gap_rwashare gap_yes gap_topseven gap_100bn gap_10bn gap_small, i(scenario) j(target)
keep scenario target gap gap_share gap_rwashare gap_yes gap_topseven gap_100bn gap_10bn gap_small

preserve
keep scenario target gap gap_share gap_rwashare gap_yes
save "$location/output/tables/capgap_point", replace
restore
